
/* ***************************************************
	FILE:  CustomMenuLoader.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.logo.app;


// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.security.CodeSource;
import java.net.URL;
import java.util.zip.ZipInputStream;

/**
	The MENU_NAMES are loaded from a zip file.
	
	Each folder is a menu
	
	Some folders will have a special file in them that denotes that the menu is to b password protected...
	
	If any file in the folder is named "password.password"
	
	Then that menu is going to be password protected with the password found as the contents of that file.  (the file is plain text)
	
	----
	since the damn JMenuBar does not allow removing MENU_NAMES and likewise the Menu does not allow removeing MenuItems, we must maintain a complete menu structure here and the listeners completely rebuild the whole menubar and set it in the frame...
	
//:There are three listener types:
	1) listener to put the file contents into the PROCEDURES AREA
	2) listener to make the password protected menu get shown
	3) listener to make the shown password protected menu to get shrunk
	
//:There are two Hashtables:
	There are a menus that are stored in a hashtable as keys, the values are to denote wether they are passwordable.
		if they are the value is "PASSWORDABLE"
		if they are not the value is ""
		
	The MENU_NAMES are also stored in another hashtable as keys with values denoting what the expand collapse state is.
		All non password MENU_NAMES will have value of:             "EXPANDED"
		All passworded MENU_NAMES that have been unlocked have:     "EXPANDED"
		All passworded MENU_NAMES that have not been unlocked have: "SHRUNK"
	
*/
public class CustomMenuLoader
{
	
	private static final String UNLOCK = "--Unlock--";
	private static final String LOCK   = "--Lock--";
	
	private final Map<String, JMenu> _folderName_menu_map = new HashMap<>();
	private final Map<JMenu, java.util.List<JMenuItem>> _menu_passworded_map = new HashMap<>();
	
	
	public CustomMenuLoader(){
	}
	

	public void makeExtensionMenus( final JMenuBar mb, final GuiConsole guiConsole ) {
		// we can hardcode "/" because that IS ZIP SEPARATOR
		
		java.util.List<ZipEntry> entries = new java.util.ArrayList<>();
		CodeSource src = CustomMenuLoader.class.getProtectionDomain().getCodeSource();
		
		if(src==null)return;
		URL jar = src.getLocation();
		
		try (ZipInputStream zip = new ZipInputStream(jar.openStream())) {
		
			while(true) {
				ZipEntry e = zip.getNextEntry();
				if (e == null)
					break;
				String name = e.getName();
				
				if (name.startsWith("deployment/XM/")) {
					/* Do something with this entry. */
					//...
					
					entries.add(e);
				}
			}
			
			for( ZipEntry ze : entries) {
				final String entryName = ze.toString();
				final String resourcePath = "/" + entryName;
				if( !ze.isDirectory() ) {
					String s = entryName;
					
					// chop off the "deployment/XM/"
					s = s.substring( "deployment/XM/".length(), s.length() );
					
					
					int i = s.lastIndexOf( "/" );
					String folder = "";
					
					if( i < s.length() && i >= 0 ) {
						folder = s.substring( 0, i );
						s = s.substring( i+1, s.length() );
						
						JMenu menu = _folderName_menu_map.get(folder);
						if( menu == null ) {
							//menu = new JMenu( folder );
							menu = new JLongMenu( folder );
							mb.add( menu );
							_folderName_menu_map.put(folder, menu);
						}
						
						if( s == null ) {
							continue;
						}
						
						if(s.equals("password.password")||s.equals("password.txt")) {
							// MAKE THE MENU PASSWORD PROTECTED AND DON'T ADD THIS ONE..
							// Wed Feb 01 21:48:32 CST 2006, made it trim off any extraneous return chars...
							final String password = (""+LogoAid.getTextFromResourcePath(resourcePath)).trim();
							
							
							final JMenuItem mi = new JMenuItem( UNLOCK );
							final String gameName = s;
							final String unlockWhat = folder;
							final JMenu unlockableMenu = menu;
							mi.addActionListener(event -> {
								if( mi.getText().equals( UNLOCK ) ) {
									final String userPassword = PasswordUI.aquirePasswordFromUser(guiConsole.getJFrame(),"Enter the password to unlock " + unlockWhat +".");
									final boolean match=userPassword.equals(password);
									
									
									if( match ) {
										putAllMenuItemsBackIntoMenu(unlockableMenu,guiConsole);
										mi.setText(LOCK);
									}
								} else {
									pullOutAllMenuItems(unlockableMenu);
									mi.setText(UNLOCK);
								}
								//action.execute();
							});
							menu.insert( mi,0 );
							
							_menu_passworded_map.put(menu, new Vector());
						} else {
							JMenuItem mi = new JMenuItem( s );
							final String gameName = s;
							mi.addActionListener( new ActionListener() { public void actionPerformed( ActionEvent event ) {
								final CustomMenuLoadAction action = new CustomMenuLoadAction(guiConsole, resourcePath);
								action.execute();
							}} );
							menu.add( mi );
						}
					}
					
				}
			}
		} catch (Exception ex) {
			//no extensions!
			final String msg = "TRIED to load extensions XM file, but failed : " + ex;
			System.out.println(msg+"<:|:>"+"virtuoso.logo.app.CustomMenuLoader.java.ux:TL-214");
			//guiConsole.putStatusMessage( msg );
		}
		
		// xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx 
		// for all passworded menus, take out all the menuitems except the one that puts them all back in
		for( JMenu menu : _menu_passworded_map.keySet()) {
			pullOutAllMenuItems(menu);
		}
		// xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
	}
	private final void pullOutAllMenuItems(final JMenu menu) {
		final java.util.List<JMenuItem> menuItems = _menu_passworded_map.get(menu);
		for( int i = 0; i < menu.getItemCount(); i++ ) {
			final JMenuItem item = menu.getItem(i);
			if(!item.getText().equals(UNLOCK) && !item.getText().equals(LOCK)) {
				// ADD UNIQUE
				menuItems.remove(item);
				menuItems.add(item);
			}
		}
		
		for( JMenuItem item : menuItems) {
			item.setEnabled(false);
		}
	}
	private final void putAllMenuItemsBackIntoMenu(final JMenu menu,final GuiConsole guiConsole) {
		for(JMenuItem item : _menu_passworded_map.get(menu)) {
			item.setEnabled(true);
		}
		guiConsole.getJFrame().setJMenuBar(guiConsole.getGuiConsoleMenuBar());
	}
}