
/* ***************************************************
	FILE:  GuiConsole.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.logo.app;

// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
import javax.swing.text.BadLocationException;
import javax.swing.text.rtf.RTFEditorKit;

/**
	Console interface
	
	
*/
public class GuiConsole extends InteractiveConsole
{
	private int _defaultTurtlePanelSize = 380;
	//:LogoFile
	public LogoFile getLogoFile(){ return _aLogoFile; }
	public void setLogoFile(final LogoFile aLogoFile){ _aLogoFile = aLogoFile; }
	private LogoFile _aLogoFile;
	
	public static final int BUFFER_SIZE = 4096;
	public static final int OUTPUT_LENGTH = 400;
	public static final int HISTORY_SIZE = 20;
	
	public GuiConsoleMenuBar getGuiConsoleMenuBar() {
		return _guicmb;
	}
	private GuiConsoleMenuBar _guicmb;
	
	String _curPrompt;
	boolean _copyPrompt;
	Clipboard _clipboard;

	Font _monofont = Font.decode("Monospaced");


	public JFrame getJFrame() { return _frame; }
	JFrame _frame;
	GuiConsolePanel _gcpanel;
	public final GuiConsolePanel getConsolePanel() { return _gcpanel; }
	MenuActor _menuActor;
	Vector _buf;
	Vector _editors;
	
	GuiEditorPanel _proceduresPanel;
	
	
	//:CurrentSkillGame
		public final String getCurrentSkillGame(){return _CurrentSkillGame;}
		public final void setCurrentSkillGame(final String aCurrentSkillGame){
			
			_CurrentSkillGame=aCurrentSkillGame;
			
			
		}
		private String _CurrentSkillGame;
	
	//:GoButton
		public final JButton getGoButton(){return _GoButton;}
		public final GuiConsole setGoButton(final JButton aGoButton){
			
			_GoButton=aGoButton;
			
			return this;
		}
		private JButton _GoButton;
	
	private Dimension TUTORIAL_SIZE = new Dimension(680, 940);
	private JFrame _tutorialFrame;
	public void showTutorialFrame() {
		_tutorialFrame.setVisible(true);
		_tutorialFrame.setState(Frame.NORMAL);
	}
	
	public final GuiEditorPanel getGuiEditorPanel() {
		return _proceduresPanel;
	}
	
	public TurtlePrimitives getTurtlePrimitives() {
		//System.out.println("trying to get TurtlePrimitives"+"<:|:>"+"virtuoso.logo.app.GuiConsole.java.ux:TL-140");
		Machine machine = getMachine();
		//System.out.println("the machine is: " + machine+"<:|:>"+"virtuoso.logo.app.GuiConsole.java.ux:TL-142");
		if( machine != null ) {
			final TurtlePrimitives tprimitives = (TurtlePrimitives)machine.getPrimitiveGroup( "virtuoso.logo.lib.TurtlePrimitives" );
			//System.out.println("found TurtlePrimitives: " + tprimitives +"<:|:>"+"virtuoso.logo.app.GuiConsole.java.ux:TL-145");
			return tprimitives;
		}
		return null;
	}
	public ZZPrimitives getZZPrimitives() {
		Machine machine = getMachine();
		if( machine != null ) {
			return (ZZPrimitives)machine.getPrimitiveGroup( "virtuoso.logo.lib.ZZPrimitives" );
		}
		return null;
	}
	
	public TurtlePanel getTurtlePanel() {
		return _turtlePanel;
	}
	TurtlePanel _turtlePanel;
	public TurtleDisplayThread _turtleDisplayThread;
	
	/**
		Create a GuiConsole
	*/
	public GuiConsole() {
		setLogoFile( new LogoFile( "" ) );
		setCurrentSkillGame( "go" );
	}
	/**
		Set up console
	*/
	protected void setup() throws SetupException {
		try {
			_clipboard = new Clipboard("private");
			_menuActor = new MenuActor( this );
			
			makeGuiPanelAndShowInFrame();
		}
		catch( Throwable e ) {
			throw new SetupException("Unable to set up graphical console: " + e.toString());
		}
		_buf = new Vector();
		_editors = new Vector();
		_curPrompt = new String();
		_copyPrompt = false;
		
		putStatusMessage("PowerMath Logo graphical console and editor v1.0");
		
		getConsolePanel().requestFocus();
	}
	void makeGuiPanelAndShowInFrame() throws LanguageException {
		_frame = new GuiConsoleWind();
		_frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		
		//	public void show() {
		//	
		//	}
		//};
		JFrame f = _frame;
		//f.setTitle("Command Center");
		f.getContentPane().setLayout(new BorderLayout());
		
		GuiConsoleFocusListener focusListener=new GuiConsoleFocusListener();
		_gcpanel=new GuiConsolePanel(this,focusListener);
		
		_proceduresPanel = new GuiEditorPanel( this, "" );
		
		
		getGuiEditorPanel().setJFrame( _frame );
		//GEMAL gemal = new GEMAL();
		//gemal.setEditorPanel( getGuiEditorPanel() );
		//gemal.setJFrame( f );
		//JMenuBar edmb = new GuiEditorMenuBar( this, gemal );
		
		
		// ===========================
		// ADD THE KEYMAPPINGS
		// SHOW_IMAGE 100 "app-images" "images/turtletracksimages/uicommands.jpg" 
		final KeyStroke FLIP_WINDOWS_KEYSTROKE = KeyStroke.getKeyStroke( KeyEvent.VK_F, java.awt.Event.CTRL_MASK );
		final KeyStroke COMMAND_CENTER_KEYSTROKE = KeyStroke.getKeyStroke( KeyEvent.VK_J, java.awt.Event.CTRL_MASK );
		final KeyStroke GRAPHICS_WINDOW_KEYSTROKE = KeyStroke.getKeyStroke( KeyEvent.VK_K, java.awt.Event.CTRL_MASK );
		final KeyStroke PROCEDURES_KEYSTROKE = KeyStroke.getKeyStroke( KeyEvent.VK_L, java.awt.Event.CTRL_MASK );
		final KeyStroke INTERRUPT_KEYSTROKE = KeyStroke.getKeyStroke( KeyEvent.VK_M, java.awt.Event.CTRL_MASK );
		// ALL INSTANCES SHARE SAME KEYMAP
		javax.swing.text.Keymap keymap = new JTextArea().getKeymap();
		
		final Action FLIP_ACTION = new AbstractAction() {
			public void actionPerformed( final ActionEvent e ) {
				new FlipAction( GuiConsole.this ).execute();
			}
		};
		final Action FOCUS_COMMAND_CENTER_ACTION = new AbstractAction() {
			public void actionPerformed( final ActionEvent e ) {
				getConsolePanel().requestFocus();
			}
		};
		final Action FOCUS_GRAPHICS_ACTION = new AbstractAction() {
			public void actionPerformed( final ActionEvent e ) {
				getTurtlePanel().requestFocus();
			}
		};
		final Action FOCUS_PROCEDURES_ACTION = new AbstractAction() {
			public void actionPerformed( final ActionEvent e ) {
				getGuiEditorPanel().requestFocus();
			}
		};
		final Action INTERRUPT_ACTION = new AbstractAction() {
			public void actionPerformed( final ActionEvent e ) {
				new StopAction(GuiConsole.this).setShouldPrintMessage(true).execute();
			}
		};
		final Action GO_ACTION = new AbstractAction() {
			public void actionPerformed( final ActionEvent e ) {
				new GoAction(GuiConsole.this).execute();
			}
		};
		
		keymap.addActionForKeyStroke( FLIP_WINDOWS_KEYSTROKE, FLIP_ACTION );
		keymap.addActionForKeyStroke( INTERRUPT_KEYSTROKE, INTERRUPT_ACTION );
		
		keymap.addActionForKeyStroke( COMMAND_CENTER_KEYSTROKE, FOCUS_COMMAND_CENTER_ACTION );
		keymap.addActionForKeyStroke( GRAPHICS_WINDOW_KEYSTROKE, FOCUS_GRAPHICS_ACTION );
		keymap.addActionForKeyStroke( PROCEDURES_KEYSTROKE, FOCUS_PROCEDURES_ACTION );
		
		// ===========================
		
		
		final Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		final int PADDING = 200;
		
		try {
			// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
			// if the screen is larger than 800 x 600, can make the turtle panel bigger
			if( screenSize.width - PADDING >= 800 && screenSize.height - PADDING >= 600 ) {
				// simple way to make bigger
				// want at least 350 for history, but min size of turtle area is 380
				_defaultTurtlePanelSize = Math.max(screenSize.height - PADDING - 350, _defaultTurtlePanelSize);
			}
			// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
		
			_turtlePanel = new TurtlePanel( _defaultTurtlePanelSize/2, _defaultTurtlePanelSize/2 );
			_turtleDisplayThread = new TurtleDisplayThread();
			_turtleDisplayThread.setTurtlePanel( _turtlePanel );
			_turtlePanel.setTurtleDisplayThread( _turtleDisplayThread );
			_turtleDisplayThread.start();
			
			_turtlePanel.setSize( _defaultTurtlePanelSize, _defaultTurtlePanelSize );
			_turtlePanel.setPreferredSize( new Dimension( _defaultTurtlePanelSize, _defaultTurtlePanelSize ) );
			_turtlePanel.setMinimumSize( new Dimension( _defaultTurtlePanelSize, _defaultTurtlePanelSize ) );
		}
		catch( InternalError e ) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.app.GuiConsole.java.ux:TL-296"+" "+"WHAT IS THIS" + e );
				}
				else {
					throw new LanguageException("WHAT IS THIS" + e );
				}
		}
		
		// *************************
		final JPanel sliderHolder = new JPanel();
		sliderHolder.setLayout(new BorderLayout());
		
		JComponent stopButton = UIAid.makeStopButton( this, INTERRUPT_ACTION );
		JButton goButton = UIAid.makeGoButton( this, GO_ACTION );
		setGoButton( goButton );
		
		JPanel p = GuiConsoleLayout.makeLayoutComponent( goButton, stopButton, sliderHolder, _turtlePanel, _gcpanel, getGuiEditorPanel() );
		f.getContentPane().add( p, BorderLayout.CENTER );
		// *************************
		
		GuiConsoleActionListener actionListener=new GuiConsoleActionListener(_gcpanel);
		actionListener.setFrame( f );
		_guicmb = new GuiConsoleMenuBar( this, actionListener, focusListener );
		f.setJMenuBar(_guicmb);
		
		f.addWindowListener(new WindowHandler(this));
		
		f.pack();
		
		
		
		// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
		// MAKE THE WINDOW GROW IF THE SCREEN IS GREATER THAN 800 x 600
		if( screenSize.width - PADDING >= 800 && screenSize.height - PADDING >= 600 ) {
			f.setSize( screenSize.width - PADDING, screenSize.height - PADDING );
		}
		// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
		
		
		
		final int fx = f.getBounds().x;
		final int fy = f.getBounds().y;
		final int fw = f.getBounds().width;
		final int fh = f.getBounds().height;
		//System.out.println( " f.getContentPane().getHeight(): " + f.getContentPane().getHeight() );
		//System.out.println( " f.getContentPane().getWidth(): " + f.getContentPane().getWidth() );
		f.setBounds( fx, fy, fw + 40, fh + 80 );
		//System.out.println( " f.getContentPane().getHeight(): " + f.getContentPane().getHeight() );
		//System.out.println( " f.getContentPane().getWidth(): " + f.getContentPane().getWidth() );
		
		
		
		// =================================================
		// GET RID OF THE SPLASH SCREEN AFTER A WHILE
		if( virtuoso.logo.app.Gui.SPLASH_SCREEN != null ) {
			while( System.currentTimeMillis() < virtuoso.logo.app.Gui.SPLASH_DISAPPEAR_TIME ) {
				try {
					Thread.currentThread().sleep( 200 );
				} catch(Exception e ) {
					// no big deal
				}
			}
			virtuoso.logo.app.Gui.SPLASH_SCREEN.setVisible(false);
			//virtuoso.logo.app.Gui.SPLASH_SCREEN.dispose();
		}
		// =================================================
		
		//_frame.pack();
		_frame.setLocationRelativeTo(null);
		f.setVisible(true);
		
		
		_gcpanel.requestFocus();
		
		_turtlePanel.init();
		try {
			TurtlePrimitives tprimitives = getTurtlePrimitives();
			if( tprimitives != null ) {
				tprimitives.setTurtlePanel(_turtlePanel);
				tprimitives.setTurtleDisplayThread(_turtleDisplayThread);
				tprimitives.updateTurtle();
				
				// ##########
				// ADD IN THE THROTTLE SLIDER
				final BoundedRangeModel rangeModel = tprimitives.getThrottleBoundedRangeModel();
				final JSlider throttleSlider = new JSlider(rangeModel);
				sliderHolder.add( throttleSlider, BorderLayout.CENTER );
				
				
				sliderHolder.setBorder( new TitledBorder(" Turtle Pace: " + rangeModel.getValue() + " % ") );
				throttleSlider.addChangeListener( new ChangeListener() {
					public void stateChanged(ChangeEvent e) {
						sliderHolder.setBorder( new TitledBorder(" Turtle Pace: " + rangeModel.getValue() + " % ") );
					}
				} );
				
				// ##########
				
			}
		} catch( Throwable t ) {
			t.printStackTrace();
			System.out.println("EXCEPTION " + t +"<:|:>"+"virtuoso.logo.app.GuiConsole.java.ux:TL-396");
		}
		_gcpanel.requestFocus();
		
		
		
//		_frame.addFocusListener( new FocusAdapter() {
//			public void focusLost( FocusEvent e ) {
//				System.out.println(e+"lost focus!"+"<:|:>"+"virtuoso.logo.app.GuiConsole.java.ux:TL-404");
//				//new RuntimeException().printStackTrace();
//			}
//		} );
		
		// ======================
		// LOAD IN A SAMPLE FILE
		if( false ) {
			try {
				final File file = new File( "sample.logo" );
				final String contents=LogoAid.getTextFromFile(this, file);
				final LogoFile logoFile = new LogoFile(contents);
				logoFile.setAssociatedFile( file );
				
				getGuiEditorPanel().setText( logoFile.getBodyText() );
				setLogoFile( logoFile );
			}
			catch (IOException e) {
				putLine("Couldn't read file: " + e.toString());
			}
		}
		else {
			try {
				new NewProceduresAction(this,true).execute();
			}
			catch( Throwable e ) {
				System.out.println("Exception " + e +"<:|:>"+"virtuoso.logo.app.GuiConsole.java.ux:TL-430");
			}
			
		}
		// ======================
		
		Thread t = new Thread() {
			public void run() {
				try {
					sleep( 50 );
				}
				catch( Exception e ) {
					
				}
				_gcpanel.requestFocus();
				
				//makeFrameForStartupTutorial_rtf();
				makeFrameForStartupTutorial_pdf();
			}
		};
		t.setPriority( 3 );
		t.start();
	}
	private void makeFrameForStartupTutorial_rtf() {
		boolean rtf = false;
		try {
			final File tutorialRTF = virtuoso.logo.startups.Startups.getTutorialRTF();
			if( tutorialRTF != null ) {
				// ##########################
				final RTFEditorKit editorKit = new RTFEditorKit();
				final JEditorPane pane = new JEditorPane();
				pane.setEditorKit( editorKit );
				final java.awt.Insets insets = new java.awt.Insets(20,20,20,20);
				pane.setMargin(insets);
				pane.setEditable(false);
				//pane.setPage(tutorialRTF.toURL());
				_tutorialFrame = new JFrame("Power Math Logo Tutorial");
				final JFrame frame = _tutorialFrame;
				frame.getContentPane().setLayout(new BorderLayout());
				
				//  Load an RTF file into the editor
				try {
					FileInputStream fi = new FileInputStream( tutorialRTF );
					editorKit.read( fi, pane.getDocument(), 0 );
				} catch( FileNotFoundException e ) {
					System.out.println( "File not found" );
				} catch( IOException e ) {
					System.out.println( "I/O error" );
				} catch( BadLocationException e ) {
					System.out.println( "bad location exception:"+e);
				}
				
				final JScrollPane spane = new JScrollPane(pane);
				
				frame.getContentPane().add(spane,BorderLayout.CENTER);
				frame.pack();
				frame.setSize(TUTORIAL_SIZE);
				frame.setLocationRelativeTo(null);
				//frame.setVisible(true);
				frame.getContentPane().doLayout();
				
				rtf = true;
				// ##########################
			}
		} catch( Exception e) {
			// no biggy try fallback plan
			System.out.println("e:"+e);
			LogoAid.showTextInFrame("exception:"+e,false);
		}
		if( !rtf ) {
			LogoAid.showTextInFrame(virtuoso.logo.startups.Startups.getStandaloneTutorialText(),false);
		}
	}
	private void makeFrameForStartupTutorial_pdf() {
		boolean pdf_shown = false;
		try {
			_tutorialFrame = new JFrame("Power Math Logo Tutorial");
			final JFrame frame = _tutorialFrame;
			frame.getContentPane().setLayout(new BorderLayout());
			final View_PDF view_pdf = new View_PDF("/deployment/Tutorial.pdf");
				
			//final JScrollPane spane = new JScrollPane1`(view_pdf);
			//frame.getContentPane().add(spane,BorderLayout.CENTER);
			frame.getContentPane().add(view_pdf,BorderLayout.CENTER);
			
			frame.pack();
			frame.setSize(TUTORIAL_SIZE);
			frame.setLocationRelativeTo(null);
			//frame.setVisible(true);
			frame.getContentPane().doLayout();
			
			pdf_shown = true;
		} catch( Exception e) {
			// no biggy try fallback plan
			System.out.println("e:"+e);
			LogoAid.showTextInFrame("exception:"+e,false);
		}
		if( !pdf_shown ) {
			LogoAid.showTextInFrame(virtuoso.logo.startups.Startups.getStandaloneTutorialText(),false);
		}
	}
	
	/**
		Gets next line in buffer, or null if no lines
		@return the string
	*/
	synchronized String popBuffer() {
		if( _buf.size() == 0 ) {
			return null;
		}
		String str = (String)(_buf.firstElement());
		_buf.removeElementAt(0);
		return str;
	}
	/**
		Pushes entry into the buffer
	*/
	public synchronized void pushBuffer( String str ) {
		_buf.addElement(str);
		notifyAll();
	}
	/**
		Has the console encountered EOF?
		@return true iff eof encountered
	*/
	public synchronized boolean eof() {
		return false;
	}
	/**
		Get a line from the console. Blocks until a line is available.
		@return the string
	*/
	public synchronized String getLine() {
		String str = popBuffer();
		
		while (str == null) {
			try {
				wait();
			}
			catch (InterruptedException e) {}
			str = popBuffer();
		}
		putLine(str);
		if( str.length() > 0 ) {
			_gcpanel.insertHistory(str);
		}
		return str;
	}
	/**
		Prompt and read a command
		@param prompt the prompt
		@exception virtuoso.logo.LanguageException write not allowed, or io closed
	*/
	public synchronized String promptGetLine( char prompt ) {
		put(prompt);
		put(' ');
		
		char[] cs = new char[2];
		cs[0] = prompt;
		cs[1] = ' ';
		_curPrompt = new String(cs);
		_copyPrompt = true;
		
		_gcpanel.requestFocus();
		
		String str = popBuffer();
		while( str == null ) {
			try {
				wait();
			}
			catch (InterruptedException e) {}
			str = popBuffer();
		}
		_curPrompt = new String();
		putLine(str);
		if( str.length() > 0 ) {
			_gcpanel.insertHistory( str );
		}
		return str;
	}
	/**
		Get all available data from stream
		@param buf buffer to read into
		@return how much data was actually read
	*/
	public synchronized int getAvailable( char[] buf ) {
		String str = null;
		
		if (_buf.size() != 0) {
			str = (String)(_buf.firstElement());
			if (str.length() > buf.length-1) {
				_buf.setElementAt(str.substring(buf.length-1),0);
				str = str.substring(0, buf.length-1).concat(Machine.LINE_SEPARATOR);
			}
			else {
				_buf.removeElementAt(0);
				str = str.concat(Machine.LINE_SEPARATOR);
			}
		}
		
		if (str == null) {
			return 0;
		}
		else {
			str.getChars(0, str.length(), buf, 0);
			return str.length();
		}
	}
	/**
		Write a char to the stream
		@param c the char
	*/
	public synchronized void put(char c){
		_gcpanel.put(String.valueOf(c));
	}
	/**
		Write a string to the console, without an EOL
		@param str the string
	*/
	public synchronized void put(String str){
		_gcpanel.put(str);
	}
	/**
		Write a char array to the stream
		@param buf the buffer
		@param num number of characters
		@exception virtuoso.logo.LanguageException write not allowed, or io closed
	*/
	public synchronized void put(char[] buf,int num){
		_gcpanel.put(new String(buf, 0, num));
	}
	/**
		Write a string to the stream, terminated with a newline
		@param str the string
		@exception virtuoso.logo.LanguageException write not allowed, or io closed
	*/
	public synchronized void putLine(String str){
		_gcpanel.put(str+'\n');
	}
	/**
		Get a character from the console
		@return char the character
	*/
	public synchronized char getChar() {
		return _gcpanel.getChar();
	}
	/**
		Character available on console?
		@return true iff available
	*/
	public synchronized boolean charAvail() {
		return _gcpanel.charAvail();
	}
	/**
		Request a new editor
		@param thread the thread running
		@param data initial data for editor window
		@exception virtuoso.logo.LanguageException failed to open editor
	*/
	public synchronized void createEditor( String data) throws LanguageException {
		_editors.addElement(new GuiEditorWind( this, data));
	}
	/**
		Close down the console
	*/
	protected void exiting() {
		int i;
		for (i=0; i<_editors.size(); i++) {
			((GuiEditorWind)(_editors.elementAt(i))).dispose();
		}
		_frame.dispose();
		_tutorialFrame.dispose();
	}
	
}