
/* ***************************************************
	FILE:  GuiConsoleActionListener.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.logo.app;

// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

/*
	Mon Jul 11 23:31:26 CDT 2005,
		pulled from GuiConsoleWind
*/

public class GuiConsoleActionListener
implements ActionListener
{

	public GuiConsoleActionListener( GuiConsolePanel gcpanel ) {
		_gcpanel = gcpanel;
	}
	private GuiConsolePanel _gcpanel;
	
	public void setFrame( final JFrame f ) {
		_f = f;
	}
	private JFrame _f;
	// =======================================
	public void actionPerformed( final ActionEvent event) {
		String cmd = event.getActionCommand();
		
		if (cmd.equals("QUIT")) {
			final boolean proceed = new HandleModifiedProceduresAction(_gcpanel._guiConsole).execute();
			if( proceed ) {
				int result = MessageWind.execute( MessageWind.genStrings("Are you sure you want to quit PowerMath Logo?"), MessageWind.genStrings("Quit", "Cancel"), _f);
				if (result == 0) {
					_gcpanel._guiConsole.goodbye();
					_gcpanel._guiConsole.mach().terminateAllThreads();
					_gcpanel._guiConsole.pushBuffer(new String());
				}
			}
		}
		else if (cmd.equals(MenuCommands.CLEAR_HISTORY)) {
			_gcpanel._historyArea.setText(_gcpanel._guiConsole._curPrompt);
			//_gcpanel._historyLength = _gcpanel._guiConsole._curPrompt.length();
		}
		
		Component rawfoc = _f.getFocusOwner();
		if (!(rawfoc instanceof JTextArea)) {
			return;
		}
		JTextArea foc = (JTextArea)rawfoc;
		if (foc == null) {
			return;
		}
		
		if (cmd.equals(MenuCommands.UNDO)) {
		}
		else if (cmd.equals(MenuCommands.CUT)) {
			if (foc == _gcpanel._entryArea) {
				int selStart = foc.getSelectionStart();
				int selEnd = foc.getSelectionEnd();
				StringSelection sel = new StringSelection( foc.getText().substring(selStart, selEnd));
				try {
					_f.getToolkit().getSystemClipboard().setContents(sel, sel);
				}
				catch (SecurityException e) {
					_gcpanel._guiConsole._clipboard.setContents(sel, sel);
				}
				foc.replaceRange(new String(), selStart, selEnd);
				foc.setCaretPosition(selStart);
				foc.select(selStart, selStart);
			}
		}
		else if (cmd.equals(MenuCommands.COPY)) {
			StringSelection sel = new StringSelection(foc.getSelectedText());
			try {
				_f.getToolkit().getSystemClipboard().setContents(sel, sel);
			}
			catch (SecurityException e) {
				_gcpanel._guiConsole._clipboard.setContents(sel, sel);
			}
		}
		else if (cmd.equals(MenuCommands.PASTE)) {
			if (foc == _gcpanel._entryArea) {
				try {
					String toPaste;
					try {
						Transferable trans = _f.getToolkit().getSystemClipboard().getContents(this);
						toPaste = (String)(trans.getTransferData(DataFlavor.stringFlavor));
					}
					catch (SecurityException e) {
						Transferable trans = _gcpanel._guiConsole._clipboard.getContents(this);
						toPaste = (String)(trans.getTransferData(DataFlavor.stringFlavor));
					}
					int selStart = foc.getSelectionStart();
					int afterPos = selStart + toPaste.length();
					foc.replaceRange(toPaste, selStart, foc.getSelectionEnd());
					foc.setCaretPosition(afterPos);
					foc.select(afterPos, afterPos);
				}
				catch (UnsupportedFlavorException e) {}
				catch (IOException e) {}
			}
		}
		else if (cmd.equals(MenuCommands.CLEAR_HISTORY)) {
			if (foc == _gcpanel._entryArea) {
				int pos = foc.getSelectionStart();
				foc.replaceRange(new String(), pos, foc.getSelectionEnd());
				foc.setCaretPosition(pos);
				foc.select(pos, pos);
			}
		}
	}
	
}