
/* ***************************************************
	FILE:  GuiConsolePanel.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.logo.app;

// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


/**
	Sun Jul 17 18:19:18 CDT 2005
		converted GuiConsoleWind to a Panel
	Sun Jul 17 22:58:44 CDT 2005
		converted to JPanel
*/
public final class GuiConsolePanel extends JPanel
{
	public boolean hasFocus() { return _entryArea.hasFocus(); }
	public void requestFocus() {
		_entryArea.requestFocus();
	}
	
	/**
		try to execute the command 
	*/
	public void hack_ConsoleInputSetText( final String text ) {
		_entryArea.setText( text );
	}

	final Vector _historyStrings = new Vector();
	
	JTextArea _historyArea;
	JScrollPane _historyScrollPane;
	JTextArea _entryArea;
	
	Vector _history;
	
	GuiConsole _guiConsole;
	public int _historyPos = -1;
	
	GuiConsolePanel( final GuiConsole console, GuiConsoleFocusListener focusListener ) {
		_guiConsole = console;
		_history = new Vector(_guiConsole.HISTORY_SIZE+2);
		
		GuiConsoleKeyListener keyListener=new GuiConsoleKeyListener(this);
		setLayout(new BorderLayout());
		
		_historyArea = new JTextArea(24, 80);
		_historyArea.setEditable(false);
		_historyArea.setBackground(Color.white);
		_historyArea.setFont(_guiConsole._monofont);
		_historyArea.setLineWrap( true );
		_historyScrollPane = new JScrollPane( _historyArea );
		//_historyScrollPane.setBorder( new TitledBorder( "Commands History" ) );
		add(_historyScrollPane, BorderLayout.CENTER);
		
		_entryArea = new JTextArea(1, 80);
		_entryArea.setBackground(Color.white);
		_entryArea.addFocusListener(focusListener);
		_entryArea.addKeyListener(keyListener);
		_entryArea.setFont(_guiConsole._monofont);
		_entryArea.setBorder( new TitledBorder( "Enter Command" ) );
		add(_entryArea, BorderLayout.SOUTH);
		
		setPreferredSize( new Dimension( 400, 110 ) );
		setMinimumSize( new Dimension( 400, 110 ) );
	}
	/**
		Crops history to prespecified length by killing lines at top
	*/
	private final StringBuffer BUFFER = new StringBuffer();
	private void cropHistory(){
		while( _historyStrings.size() > _guiConsole.OUTPUT_LENGTH ) {
			_historyStrings.removeElementAt( 0 );
		}
		
		BUFFER.setLength(0);
		final int count = _historyStrings.size();
		for( int i = 0; i < count; i++ ) {
			BUFFER.append( _historyStrings.elementAt( i ) );
		}
		_historyArea.setText( BUFFER.toString() );
		_historyArea.setCaretPosition(BUFFER.toString().length() );
	}
	/**
	 * Insert a line into the input history
	 * @param str the string to insert
	 */
	void insertHistory(String str){
		_history.insertElementAt(str, 0);
		if (_history.size() > _guiConsole.HISTORY_SIZE){
			_history.removeElementAt(_history.size() - 1);
		}
	}
	/**
	 * Write a string to the console
	 * @param str the string
	 */
	void put(String str){
		boolean entryFocus = _entryArea.hasFocus();
		
		if( _guiConsole._curPrompt.length() > 0 ){
			if( _guiConsole._copyPrompt ) {
				_historyStrings.addElement( "\n"+str+_guiConsole._curPrompt );
				_guiConsole._copyPrompt = false;
			}
			else{
				_historyStrings.addElement( str );
			}
		}
		else{
			_historyStrings.addElement( str );
		}
		
		cropHistory();
		// ================================
		// Sun Jul 17 23:40:29 CDT 2005, changed when move to swing occurred
		//_historyArea.setCaretPosition(_historyLength+1);
		if( entryFocus ) {
			_entryArea.requestFocus();
		}
		// ================================
	}
	/**
	 * Get a character from the console
	 * @return char the character
	 */
	char getChar(){
		
		String str = _entryArea.getText();
		/*
		while( str.length() == 0 ){
			try {
				Thread.currentThread().yield();
				//wait();
			}
			catch( Exception e ) {
				System.out.println("e:"+e+"<:|:>"+"virtuoso.logo.app.GuiConsolePanel.java.ux:TL-195");
			}
			str = _entryArea.getText();
		}
		*/
		if( str.length() == 0 ) {
			return '\0';
		}
		_entryArea.setText(str.substring(1));
		return str.charAt(0);
		
	}
	/**
	 * Character available on console?
	 * @return true iff available
	 */
	boolean charAvail() {
		return (_entryArea.getText().length() > 0);
	}
}