
/* ***************************************************
	FILE:  GuiEditorPanel.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.logo.app;

// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
/**
	Mon Jul 18 01:31:16 CDT 2005
		pulled from GuiEditorWind
*/
public class GuiEditorPanel extends JPanel
{
	public void setText( final String text ) {
		_textArea.setText( text );
	}
	public String getText() {
		return _textArea.getText();
	}

	public void requestFocus(){_textArea.requestFocus();}
	public boolean hasFocus(){return _textArea.hasFocus();}

	GuiConsole _guiConsole;
	private JTextArea _textArea;
	private JScrollPane _scrollPane;
	
	public void setJFrame( final JFrame frame ) { _frame = frame; }
	public JFrame getJFrame() { return _frame; }
	private JFrame _frame;
	
	GuiEditorPanel( final GuiConsole console, String data ) {
		_guiConsole = console;
		setLayout( new GridBagLayout() );
		
		_textArea = new SimpleTextEditor(data, 30, 40 ) {
			public void paint( Graphics g ) {
				
				
				final Color c = g.getColor();
				g.setColor( Color.white );
				
				g.fillRect( 0, 0, getWidth()-1, getHeight()-1 );
				g.setColor( c );
				
				
				Image image = console.getLogoFile().isDefined()
				                  ? UIAid.DEFINED_IMAGE
				                  : UIAid.UNDEFINED_IMAGE
				                  ;
				
				final int clipWidth = getParent().getBounds().width;
				final int x = clipWidth - image.getWidth( this );
				final int y = -getY();
				g.drawImage( image, x, y, this );
				
				// PAINT THE LOCK STATUS FEEDBACK
				if( _guiConsole.getLogoFile().isLocked() ) {
					final Image lockImage = UIAid.LOCK_IMAGE;
					final int lockWidth = lockImage.getWidth( this );
					final int lockHeight = lockImage.getHeight( this );
					final float scale = (float)1.4;
					final int drawLockWidth = (int)(lockWidth * scale);
					final int drawLockHeight = (int)(lockHeight * scale);
					final int PAD = -10;
//					final int lockX = x - drawLockWidth - PAD;
//					final int lockY = y;
					int lockX = clipWidth / 2;
					int lockY = getParent().getBounds().height / 2;
					lockX -= (drawLockWidth/2);
					lockY -= (drawLockHeight/2) + getY();
					
					
					g.drawImage( lockImage, lockX, lockY, drawLockWidth, drawLockHeight, this );
					
				}
				super.paint( g );
			}
		};
		_textArea.setOpaque( false );
		_textArea.setFont(_guiConsole._monofont);
		_textArea.setLineWrap( true );
		final int vPolicy = ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS;
		final int hPolicy = ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER;
		_scrollPane = new JScrollPane(_textArea,vPolicy,hPolicy);
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.CENTER;
		add(_scrollPane, gbc);
		
		
		_textArea.getDocument().addDocumentListener( new DocumentListener() {
			public void insertUpdate(DocumentEvent de){
				console.getLogoFile().setBodyText(_textArea.getText());repaint();
			}
			public void removeUpdate(DocumentEvent de){
				console.getLogoFile().setBodyText(_textArea.getText());repaint();
			}
			public void changedUpdate(DocumentEvent de){
				console.getLogoFile().setBodyText(_textArea.getText());repaint();
			}
		} );
		
	}
	
	/**
		Accept the edits, this puts the procedures text INTO THE INTERPRETER
	*/
	public void acceptEdits() {
		//final String predefinedProcedures=" \n "+virtuoso.logo.startups.Startups.getAvailableProceduresText() + " \n ";
		//InterpreterThread thread = new InterpreterThread(_guiConsole.mach(), new IOStream(new BufferedReader(new StringReader( predefinedProcedures+" " + _textArea.getText()))));
		
		InterpreterThread thread = new InterpreterThread(_guiConsole.mach(), new IOStream(new BufferedReader(new StringReader(_textArea.getText()))));
		
		// Wed Apr 14 09:20:38 CDT 2021, this priority was zero, likely none of this priority stuff is needed, but the gui console probably had not been 'run' yet, so it had no priority either.... exception was being printed to console at runtime
		//thread.setPriority(_guiConsole._priority);
		
		thread.start();
	}
	
	public boolean equals(Object obj){
		return obj == this;
	}
	
	public void setEditable(boolean editable) {
		_textArea.setEditable(editable);
	}
	
}