
/* ***************************************************
	FILE:  JMenuKludge.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.logo.app;


// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

/**
	Invoke the show(MouseEvent e) method on this popup menu. There is however one catch. The first time it shows up, it does not know its dimensions (I don't know why this is so), therefore you make approximate calculations for its size. After the first time its automatic!
	
	The popup menu is guaranteed to show up always in screen.
	
	By the way, with JDK 1.4 you can get the available size of the screen, i.e. the full screen minus the windows taskbar or any other bars that are currently active on the screen.
	
	@author Rizwan A. Qureshi
	@version 1.0
*/
 
public class JMenuKludge extends JPopupMenu
{
	private Dimension _screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	
	private static int _screenW = 0;
	private static int _screenH = 0;
	private static final int _correction = 50;//approx pixels for the taskbar
	private boolean _firstTime = true;
	private static int _width = 100;
	private static int _height = 150;
	
	public JMenuKludge() {
		_screenW = _screenSize.width;
		_screenH = _screenSize.height-_correction;
	}
	public JMenuKludge(String title) {
		super(title);
		_screenW = _screenSize.width;
		_screenH = _screenSize.height-_correction;
	}
	
	protected void show(MouseEvent e) {
		Component comp = e.getComponent();
		Point p = e.getPoint();
		Point original = e.getPoint();
		Point newPoint = this.getNewLocation(p,original,comp);
		this.show(e.getComponent(),newPoint.x, newPoint.y);
	}
	
	private Point getNewLocation(Point p,Point original,Component comp) {
		SwingUtilities.convertPointToScreen(p,comp);
		if (_firstTime) {
			this.computeMaximumStringWidth();
		}
		
		int absX = _firstTime?_screenW-_width:_screenW-getSize().width;
		int absY = _firstTime?_screenH-_height:_screenH-getSize().height;
		boolean xChanged = p.x>absX;
		boolean yChanged = p.y>absY;
		if( _firstTime ) {
			_firstTime = false;
		}
		if( !xChanged && !yChanged ) {
			return original;
		}
		p.setLocation(xChanged?absX:0,yChanged?absY:0);
		SwingUtilities.convertPointFromScreen(p, comp);
		if (xChanged && !yChanged) {
			p.setLocation(p.x,original.y);
		}
		if( !xChanged && yChanged ) {
			p.setLocation(original.x,p.y);
		}
		return p;
	}
	
	//compute widht/height of Popupmenu only for the first time
	private void computeMaximumStringWidth() {
		Component[] menuItems = this.getComponents();
		int maxWidth = 0;
		for (int i=0;i<menuItems.length;i++){
			if (menuItems[i] instanceof JMenuItem item){
				String text = item.getText();
				if (text==null) {
					continue;
				}
				int len = SwingUtilities.computeStringWidth(item.getFontMetrics(item.getFont()),text)+60;
				if (len > maxWidth) {
					maxWidth = len;
				}
			}
		}
		_width = maxWidth;
		_height = menuItems.length*18;//height of one menuItem is around 20 pixels (approx)
	}
	
}