
/* ***************************************************
	FILE:  HandleModifiedProceduresAction.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.logo.app.actions;

// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

public class HandleModifiedProceduresAction
{
	public boolean execute() {
		return execute_util( 0 );
	}
	public boolean execute_lockCase() {
		return execute_util( 1 );
	}
	
	/**
		returns false if the user chose 'cancel'
		returns true in all other cases
	*/
	private boolean execute_util( final int type ) {
		// maybe ask user to save prior work...
		final LogoFile logoFile = _guiConsole.getLogoFile();
		if( logoFile.isModified() ) {
			// ==========================================
				// IF THE FILE IS NULL, THEN WE ARE FROM A NEW,
				// SO IN THIS CASE IT WOULD FUNCTION LIKE A SAVE_AS
				// BECAUSE THE USER NEEDS TO CHOOSE A FILE NAME
				final File file = logoFile.getAssociatedFile();
			//
			// IF THE FILE IS NOT NULL, THEN THE SAVE OCCURS TO THAT FILE
			// BUT IN EITHER CASE WE ASK THE USER IF THEY EVEN WANT TO SAVE...
			final String[] messages = new String[]{"The procedure contents have changed, would you like to save?"};
			
			final boolean noOverwrite = logoFile.isLocked() && type != 1;
			final String save = (file==null||noOverwrite)?"Save As...":"Save";
			
			final String[] buttons = new String[]{ save, "Discard", "Cancel" };
			int result = MessageWind.execute( messages, buttons, _guiConsole.getJFrame() );
			
			//System.out.println("result " + result +"<:|:>"+"virtuoso.logo.app.actions.HandleModifiedProceduresAction.java.ux:TL-99");
			
			if( result == 0 ) {// SAVE
				if( file == null || noOverwrite ) {  // no file yet
					new SaveProceduresAsAction(_guiConsole).execute();
				}
				else { // JUST SAVE TO THE FILE
					logoFile.setBodyText( _guiConsole.getGuiEditorPanel().getText() );
					boolean success = LogoAid.saveTextToFile( _guiConsole, logoFile.getFullFileText(), file );
					
					if( !success ) {
						return true;
					}
				}
				
			}
			else if( result == 1 ) { // DISCARD
				// don't save or anything we are fine
				if( type == 1 ) {
					logoFile.revertBodyText();
					LogoAid.saveTextToFile( _guiConsole, logoFile.getFullFileText(), file );
				}
			}
			else if( result == 2 ) { // CANCEL
				return false;
			}
			else {
				System.out.println("WARNING, THIS SHOULD NEVER HAPPEN!"+"<:|:>"+"virtuoso.logo.app.actions.HandleModifiedProceduresAction.java.ux:TL-126");
			}
		}
		else if( type == 1 ) {
			// IT WAS NOT MODIFIED, BUT WE ARE LOCKING SO SAVE ANYWAY
			// dont' need to ask for file...
			logoFile.setBodyText( _guiConsole.getGuiEditorPanel().getText() );
			try {
				LogoAid.saveTextToFile( _guiConsole, logoFile.getFullFileText(), logoFile.getAssociatedFile() );
			}
			catch( Exception e ) {
				// failed to lock the file
				logoFile.setIsLocked( false );
			}
		}
		return true;
	}
	public HandleModifiedProceduresAction( final GuiConsole guiConsole ) {
		_guiConsole = guiConsole;
	}
	private GuiConsole _guiConsole;
}