
/* ***************************************************
	FILE:  LoadProceduresAction.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.logo.app.actions;

// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

/**
	This thing has the behavior of the user picking a file whose contents are appended into the procedures area.
	
	The 'file' that the procedures area is associated with does not change, but the modified state of it does, so then later the user may be prompted to save their changes...
*/
public class LoadProceduresAction
{
	private GuiConsole _guiConsole;
	
	public LoadProceduresAction(final GuiConsole guiConsole) {
		_guiConsole = guiConsole;
	}
	
	public void execute() {
		try {
			new ResetMachineAction(_guiConsole).setShouldAcceptEdits(false).execute();
			
			_guiConsole.setCurrentSkillGame( "go" );
			
			final File file = LogoAid.promptUserForFile(_guiConsole, "Load Procedures", (dir, name) -> name.toLowerCase().endsWith(".logo"));
			
			if(file == null) {
				return;
			}
			String contents = LogoAid.getTextFromFile(_guiConsole, file);
			
			if(contents != null) {
				final String oldContents =_guiConsole.getGuiEditorPanel().getText();
				String sep = "";
				if( !oldContents.trim().equals( "" ) ) {
					sep = "\n";
				}
				// we make a temp logoFile to parse out the body, (for loading we are ignoring the header and using the logoFile already in place)
				final LogoFile logoFile = new LogoFile( contents );
				
				final String newBodyText = oldContents + sep + logoFile.getBodyText();
				// Wed Sep 19 20:51:28 CDT 2012, changed to NOT append
				//final String newBodyText = logoFile.getBodyText();
				
				_guiConsole.getLogoFile().setBodyText(newBodyText);
				
				_guiConsole.getGuiEditorPanel().setText(newBodyText);
				_guiConsole.getGuiEditorPanel().requestFocus();
				_guiConsole.getGuiEditorPanel().repaint();
				
				//new FlipAction( _guiConsole ).execute();
				
				new ResetMachineAction(_guiConsole).execute();
			}
		} catch (IOException ex) {
			_guiConsole.putLine("Couldn't read file: " + ex);
			_guiConsole.setCurrentSkillGame("go");
		}
	}
	
	
}