
/* ***************************************************
	FILE:  ResetMachineAction.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.logo.app.actions;

// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

public class ResetMachineAction
{
	/**
		
	*/
	public void execute() {
		new StopAction( getGuiConsole() ).setShouldClearScreen(getShouldClearScreen()).execute();
		
		getGuiConsole().getMachine().eraseAll();
		getGuiConsole().mach().terminateAllThreads();
		
		if (getShouldAcceptEdits()) {
			getGuiConsole().getGuiEditorPanel().acceptEdits();
		}
		
		if (getShouldConsoleRequestFocus()) {
			getGuiConsole().getConsolePanel().requestFocus();
		}
		
		LogoFile logoFile = getGuiConsole().getLogoFile();
		if (getShouldUpdateDefinedBodyText()) {
			if (logoFile != null) { 
				// Thu Jul 01 18:08:16 CDT 2021, there is race condition here in regards to above acceptEdits
				logoFile.setDefinedBodyText(logoFile.getBodyText());
			}
		}
		
		if (logoFile != null) { 
			getGuiConsole().getGuiEditorPanel().setEditable(!logoFile.isLocked());
		}
		
		// make sure pen is down
		getGuiConsole().getTurtlePrimitives().pPENDOWN_IMP();
		// make sure refresh is 0
		getGuiConsole().getTurtlePanel().setRedisplayInterval( 0 );
		getGuiConsole()._turtleDisplayThread.setInterval(getGuiConsole().getTurtlePanel().getRedisplayInterval());
	}
	
	public ResetMachineAction( final GuiConsole guiConsole ) {
		setGuiConsole( guiConsole );
		
		//default ShouldClearScreen
		setShouldClearScreen(true);
		
		// default ShouldAcceptEdits
		setShouldAcceptEdits(true);
		
		// default ShouldConsoleRequestFocus
		setShouldConsoleRequestFocus(true);
		
		// default ShouldUpdateDefinedBodyText
		setShouldUpdateDefinedBodyText(true);
	}
	
	//:GuiConsole
		public final GuiConsole getGuiConsole(){return _GuiConsole;}
		public final ResetMachineAction setGuiConsole(final GuiConsole aGuiConsole){
			
			_GuiConsole=aGuiConsole;
			
			return this;
		}
		private GuiConsole _GuiConsole;
	
	//:ShouldClearScreen
		public final boolean getShouldClearScreen(){return _ShouldClearScreen;}
		public final ResetMachineAction setShouldClearScreen(final boolean aShouldClearScreen){
			
			_ShouldClearScreen=aShouldClearScreen;
			
			return this;
		}
		private boolean _ShouldClearScreen;
	
	//:ShouldPrintMessage
		public final boolean getShouldPrintMessage(){return _ShouldPrintMessage;}
		public final ResetMachineAction setShouldPrintMessage(final boolean aShouldPrintMessage){
			
			_ShouldPrintMessage=aShouldPrintMessage;
			
			return this;
		}
		private boolean _ShouldPrintMessage;
	
	//:ShouldAcceptEdits
		public final boolean getShouldAcceptEdits(){return _ShouldAcceptEdits;}
		public final ResetMachineAction setShouldAcceptEdits(final boolean aShouldAcceptEdits){
			
			_ShouldAcceptEdits=aShouldAcceptEdits;
			
			return this;
		}
		private boolean _ShouldAcceptEdits;
	
	//:ShouldConsoleRequestFocus
		public final boolean getShouldConsoleRequestFocus(){return _ShouldConsoleRequestFocus;}
		public final ResetMachineAction setShouldConsoleRequestFocus(final boolean aShouldConsoleRequestFocus){
			
			_ShouldConsoleRequestFocus=aShouldConsoleRequestFocus;
			
			return this;
		}
		private boolean _ShouldConsoleRequestFocus;
	
	//:ShouldUpdateDefinedBodyText
		public final boolean getShouldUpdateDefinedBodyText(){return _ShouldUpdateDefinedBodyText;}
		public final ResetMachineAction setShouldUpdateDefinedBodyText(final boolean aShouldUpdateDefinedBodyText){
			
			_ShouldUpdateDefinedBodyText=aShouldUpdateDefinedBodyText;
			
			return this;
		}
		private boolean _ShouldUpdateDefinedBodyText;
	
}