
/* ***************************************************
	FILE:  LogoFile.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.logo.file;

// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
/**
	Logo files can optionally have a header and a body.
	
	If the file is just normal text, there is no header.
	
	However, if the file has a header, those header lines will all be located at the beginning of the file, and all those header lines will start with the prefix of:
	
	<<LOGO_HEADER>>
	
	This makes the separation easy because we just iterate over the lines and check for the prefix.
	
*/
public class LogoFile
{
	//:LogoFileHeader
	public LogoFileHeader getLogoFileHeader(){ return _aLogoFileHeader; }
	public void setLogoFileHeader(final LogoFileHeader aLogoFileHeader){ _aLogoFileHeader = aLogoFileHeader; }
	private LogoFileHeader _aLogoFileHeader;
	//:LogoFileBody
	public LogoFileBody getLogoFileBody(){ return _aLogoFileBody; }
	public void setLogoFileBody(final LogoFileBody aLogoFileBody){ _aLogoFileBody = aLogoFileBody; }
	private LogoFileBody _aLogoFileBody;
	
	public boolean isModified() { return _isModified; }
	private boolean _isModified;
	
	public boolean isDefined() { return _isDefined; }
	private boolean _isDefined;
	
	public LogoFile( final String fileText ) {
		init( fileText );
	}
	
	private File _associatedFile = null;
	public File getAssociatedFile() { return _associatedFile; }
	public void setAssociatedFile( final File associatedFile ) { _associatedFile = associatedFile; }
	
	private String _initialText = "";
	public String getInitialText() { return _initialText; }
	
	public void revertBodyText() {
		init( getInitialText() );
	}
	
	private String _initialHeaderText = "";
	public String getInitialHeaderText() { return _initialHeaderText; }
	
	private String _initialBodyText = "";
	public String getInitialBodyText() { return _initialBodyText; }
	
	private String _definedBodyText = "";
	public String getDefinedBodyText() { return _definedBodyText; }
	public void setDefinedBodyText( final String text ) {
		_definedBodyText = text;
		_isDefined = getDefinedBodyText().equals( getBodyText() );
	}
	
	/**
		Split the file text into the HEADER and the BODY.
	*/
	public void init( final String fileText ) {
		setLogoFileHeader( new LogoFileHeader() );
		setLogoFileBody( new LogoFileBody() );
		
		_initialText += fileText;
		
		final StringBuffer headerBuffer = new StringBuffer();
		final StringBuffer bodyBuffer = new StringBuffer();
		boolean isHeaderPossible = true;
		// ONLY THE FIRST LINES CAN BE PART OF HEADER
//:----------
	try {
		final BufferedReader reader = new BufferedReader(new StringReader(fileText));
		String line = null;
		while( null != (line = reader.readLine()) ) {
			if( isHeaderPossible && line.indexOf( LogoFileHeader.HEADER_LINE_PREFIX ) == 0 ) {
				headerBuffer.append( line ).append( "\n" );
			}
			else {
				isHeaderPossible = false;
				bodyBuffer.append( line ).append( "\n" );
			}
			//buffer.append(line).append( "\n" );
		}
		reader.close();
	}
	catch( Exception e ) {
		System.out.println("Exception " + e +"<:|:>"+"virtuoso.logo.file.LogoFile.java.ux:TL-152");
	}
//:----------
//		final StringTokenizer tokenizer = new StringTokenizer( fileText, "\n" );
//		while( tokenizer.hasMoreElements() ) {
//			final String line = tokenizer.nextToken();
//			if( isHeaderPossible && line.indexOf( LogoFileHeader.HEADER_LINE_PREFIX ) == 0 ) {
//				headerBuffer.append( line + "\n" );
//			}
//			else {
//				isHeaderPossible = false;
//				bodyBuffer.append( line + "\n" );
//			}
//		}
		
		_initialHeaderText = headerBuffer.toString();
		_initialBodyText = bodyBuffer.toString();
		
		getLogoFileHeader().setText( _initialHeaderText );
		_isLocked = getLogoFileHeader().isLocked();
		setBodyText( _initialBodyText );
		
		
		
		
		
		
		
	}
	
	/**
		Used to swap out the body text.
		
		(from the editor panel)
	*/
	public void setBodyText( final String text ) {
		
		getLogoFileBody().setText( text );
		_isModified = !getInitialBodyText().equals( text );
		_isDefined = getDefinedBodyText().equals( text );
	}
	public String getBodyText() {
		return getLogoFileBody().getText();
	}
	
	public boolean isLocked() { return _isLocked; }
	public void setIsLocked( final boolean locked ) {
		if( _isLocked == locked ) {
			return;
		}
		_isLocked = locked;
		if( !locked ) {
			// remove the lock line
			getLogoFileHeader().removeLock();
		}
		else {
			// add the lock line
			getLogoFileHeader().addLock();
		}
		
		
	}
	private boolean _isLocked;
	
	
	/**
		returns the header and the body
	*/
	public String getFullFileText() {
		return getLogoFileHeader().getText() + getLogoFileBody().getText();
	}
	
}