
/* ***************************************************
	FILE:  HelpDictionary.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.logo.lib;



// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
import java.util.*;


public class HelpDictionary
{

	private Hashtable _table = new Hashtable();

	public void add(final String s) {
		final StringTokenizer tokenizer = new StringTokenizer(s, " ");
		if( tokenizer.hasMoreTokens() ) {
			final String tok = tokenizer.nextToken();
			_table.put( tok, s.substring( tok.length(), s.length() ) );
		}
	}
	public void ABREVIATE( final String s, final String synonym ) {
		String help = (String)_table.get(s);
		if( help != null ) {
			_table.put( synonym, help );
		}
	}
	public String getHelp(String command) {
		if( command == null ) {
			return "";
		}
		command = command.toUpperCase();
		if( command.equals( "INDEX" ) ) {
			return getIndex();
		}
		String help = (String)_table.get(command);
		if( help == null ) {
			return "There is no help for "+command+".";
		} else {
			return command + help;
		}
	}
	private String getIndex() {
		final StringBuffer buffer = new StringBuffer();
		
		for( Enumeration keys = _table.keys(); keys.hasMoreElements(); ) {
			final String key = (String)keys.nextElement();
			buffer.append( key );
			if( keys.hasMoreElements() ) {
				buffer.append( ", " );
			}
		}
		return buffer.toString();
	}
		
	public HelpDictionary() {
		
		add("XSIZE returns the current xmax value for the graphics window. X-coordinates range from -XSIZE to XSIZE-1. Therefore, the width of the window is 2*XSIZE.");
		add("YSIZE returns the current ymax value for the graphics window. Y-coordinates range from -YSIZE to YSIZE-1. Therefore, the height of the window is 2*YSIZE.");
		add("CLEAN fills the entire graphics window with the current background color. The current turtle position and state are not affected.");
		add("CLEARSCREEN moves the turtle to the home position, and fills the entire graphics window with the current background color.");
		add("WRAP sets the current window border behavior to \"wrap.\" If the turtle moves past an edge of the window, it will \"wrap around\", or reappear on the other side.");
		add("WINDOW sets the current window border behavior to \"window.\" If the turtle moves past an edge of the window, it will disappear until it is moved back into view.");
		add("FENCE sets the current window border behavior to \"fence.\" Any call that attempts to move the turtle past the edge of the window will throw an error. If the mode was \"window\" and the turtle is already past the edge of the window, FENCE will throw an error.");
		add("REFRESHINTERVAL sets the refresh interval in milliseconds. That is, if set to 1000, REFRESHINTERVAL will cause the turtle screen to refresh at most once per second during drawing operations. Setting an interval of 0 causes all drawing to happen immediately. If a negative interval is specified, no drawing will happen automatically, and the REFRESH command should be used to cause the screen to update. The window will, however, always update in response to window manager events, regardless of the setting of REFRESHINTERVAL. If the argument is not an integer, REFRESHINTERVAL throws an error.");
		add("REFRESH forces an update of the turtle graphics window. This is typically used in conjunction with a negative REFRESHINTERVAL value, to cause the image to be redisplayed at specific times during the drawing process.");
		
		add("FORWARD moves the turtle forward the given number of pixels in the current heading direction. If the pen is currently down, the turtle will draw a line along its path. If the given input is not a number, FORWARD will throw an error.");
		add("BACK moves the turtle the given number of pixels in the direction opposite from the current heading direction. The heading of the turtle is not changed. If the pen is currently down, the turtle will draw a line along its path. If the given input is not a number, BACK will throw an error.");
		add("RIGHT changes the turtle's heading by the specified number of degrees to clockwise. The turtle's position is not changed. If the given input is not a number, RIGHT will throw an error.");
		add("LEFT changes the turtle's heading by the specified number of degrees to clockwise. The turtle's position is not changed. If the given input is not a number, LEFT will throw an error.");
		add("SETX moves the turtle such that its y-position remains the same but its x-position is the given value. The heading of the turtle is not changed. If the pen is currently down, the turtle will draw a line along its path. If the given input is not a number, SETX will throw an error.");
		add("SETY moves the turtle such that its x-position remains the same but its y-position is the given value. The heading of the turtle is not changed. If the pen is currently down, the turtle will draw a line along its path. If the given input is not a number, SETY will throw an error.");
		add("SETXY moves the turtle such that its x-position and y-position become the given values. The heading of the turtle is not changed. If the pen is currently down, the turtle will draw a line along its path. If the given inputs are not numbers, SETXY will throw an error.");
		add("SETPOS moves the turtle to the coordinates specified by the input position list. The input must be a two-element list, the first element being the x-coordinate and the second being the y-coordinate. The heading of the turtle is not changed. If the pen is currently down, the turtle will draw a line along its path. If the given input is not a list or does not contain exactly two numeric elements, SETPOS will throw an error.");
		add("SETHEADING sets the heading of the turtle to the given value, in degrees. A heading of 0 points straight up, and positive values run clockwise. The position of the turtle is not changed. If the given input is not a number, SETHEADING will throw an error.");
		add("HOME moves the turtle to the origin. It has the same effect as SETXY 0 0. The heading of the turtle is not changed. If the pen is currently down, the turtle will draw a line along its path.");
		add("LABEL draws the specified expression on the screen as text, starting from the current turtle position, in the same form as it would appear if it were PRINTed.");
		
		add("XCOR returns the current x-coordinate of the turtle.");
		add("YCOR returns the current x-coordinate of the turtle.");
		add("POS returns the current position of the turtle in the form of a two-element list, where the first element is the x-coordinate and the second element is the y-coordinate.");
		add("HEADING returns the current heading of the turtle, in degrees. A heading of 0 points straight up, and positive values run clockwise.");
		add("DISTANCETOXY returns the distance from the current turtle position to the given coordinates. The position and heading of the turtle are not changed. If the given inputs are not numbers, DISTANCETOXY will throw an error.");
		add("DISTANCETO returns the distance from the current turtle position to the given coordinates. The input must be a two-element list, the first element being the x-coordinate and the second being the y-coordinate. The position and heading of the turtle are not changed. If the given input is not a list or does not contain exactly two numeric elements, DISTANCETO will throw an error.");
		add("TOWARDSXY returns the heading pointing from the turtle position to the given coordinates, in degrees. A heading of 0 points straight up, and positive values run clockwise. The position and heading of the turtle are not changed. If the given inputs are not numbers, TOWARDSXY will throw an error.");
		add("TOWARDS returns the heading pointing from the turtle position to the given coordinates, in degrees. The input must be a two-element list, the first element being the x-coordinate and the second being the y-coordinate. A heading of 0 points straight up, and positive values run clockwise. The position and heading of the turtle are not changed. If the given input is not a list or does not contain exactly two numeric elements, TOWARDS will throw an error.");
		
		add("SHOWTURTLE shows the turtle if it is currently hidden. If the turtle is already shown, SHOWTURTLE has no effect.");
		add("HIDETURTLE hides the turtle if it is currently shown. If the turtle is already hidden, HIDETURTLE has no effect.");
		add("PENDOWN activates the drawing pen. Any subsequent move commands will cause the turtle to draw lines in the current pen color.");
		add("PENUP deactivates the drawing pen. The turtle will not perform any drawing during subsequent move commands.");
		add("PENERASE sets the drawing pen to erase. Any subsequent move commands will cause the turtle to draw lines in the current background color.");
		add("SETPENCOLOR sets the color of the drawing pen. The color may be in the format of a three-element list in which the elements are the RGB values, each between 0 and 255. It may also be a word specifying a color from the palette. If the input is a list with more or less than three elements or elements that are not integers between 0 and 255, or if the input is a word that does not correspond to an entry in the color palette, SETPENCOLOR throws an error.");
		add("SETBACKGROUND sets the color of the background, also the color of the eraser. The color may be in the format of a three-element list in which the elements are the RGB values, each between 0 and 255. It may also be a word specifying a color from the palette. If the input is a list with more or less than three elements or elements that are not integers between 0 and 255, or if the input is a word that does not correspond to an entry in the color palette, SETBACKGROUND throws an error.");
		add("GETPENCOLOR returns the color of the pen, in the form of a three-element list representing the RGB color. Each value in the list will be an integer between 0 and 255.");
		add("GETBACKGROUND returns the color of the background, also the color of the eraser, in the form of a three-element list representing the RGB color. Each value in the list will be an integer between 0 and 255.");
/**
      !!!!                                                  
    !!                                                      
  !!!!!!!!      !!!!!!      !!!!    !!!!  !!                
    !!          !!        !!    !!  !!  !!  !!              
    !!          !!        !!    !!  !!  !!  !!              
    !!          !!        !!    !!  !!  !!  !!              
    !!          !!          !!!!    !!  !!  !!              
                                                            
*/
		add("WAIT causes the current thread to pause for the given amount of time, measured in milliseconds. If the argument is not a nonnegative integer, WAIT throws an error.");
		
/**
        !!                                      
        !!                  !!                  
    !!!!!!    !!!!!!      !!!!!!!!    !!!!!!    
  !!    !!          !!      !!              !!  
  !!    !!      !!!!!!      !!          !!!!!!  
  !!    !!    !!    !!      !!        !!    !!  
    !!!!!!      !!!!!!        !!!!      !!!!!!  
                                                
*/
		

add("WORD concatenates the given arguments, which should be words, into a single word and returns the result. If an argument is a list, WORD throws an error.");
add("LIST creates a new list in which the given arguments are the members. The arguments can be either words or lists.");
add("SENTENCE creates a new list by concatenating the arguments. If an argument is a word, it becomes a member of the new list. If an argument is a list, its members become members of the new list.");
add("FPUT creates a new list in which the first argument is the first element, and the members of the second argument are the remaining elements. If the second argument is not a list, FPUT throws an error.");
add("LPUT creates a new list in which the first argument is the last element, and the members of the second argument are the remaining elements. If the second argument is not a list, LPUT throws an error.");
add("PARSE parses the given word as a list and returns the new list. If the given expression is not a word, or if it is badly formatted, PARSE throws an error.");
//add("QSORT sorts the given list using the QuickSort algorithm, and using the second argument as a comparison function. The second argument should be a function which returns \"TRUE if its first argument should be placed before its second argument, or \"FALSE otherwise. The lambda may either be a lambda list or a procedure or primitive name. Typically, it should be one of the primitive names \"BEFOREP or \"LESSP. If the first argument to QSORT is not a list, or its second argument is not a lambda that takes two arguments, QSORT throws an error.");
//add("GENSYM generates and returns a unique symbol of the form \".n\" where n is a unique positive integer.");
//add("Logo provides a number of list manipulation commands, with which you can parse and extract parts of words and lists.");
add("FIRST returns the first element of the given argument, if it is a list, or the first character, if it is a word. If the list or word is empty, FIRST will throw an error.");
add("LAST returns the last element of the given argument, if it is a list, or the first character, if it is a word. If the argument is empty, LAST will throw an error.");
add("BUTFIRST returns the the given argument with FIRST removed. If the argument is empty, BUTFIRST will throw an error. BUTFIRST can be abbreviated BF.");
add("BUTLAST returns the the given argument with LAST removed. If the argument is empty, BUTLAST will throw an error. BUTLAST can be abbreviated BL.");
add("ITEM returns the num'th element of the second argument. Num must be an integer. If the second argument is a list, ITEM returns the num'th element of the list. If the second argument is a word, ITEM returns the num'th character of the word. If num is not an integer, or if num is less than 1 or greater than the number of elements in the second argument, ITEM will throw an error.");
add("PICK chooses and returns a random element of the given argument. If the argument is a list, PICK returns a randomly-chosen element of the list. If the argument is a word, PICK returns a randomly-chosen character of the word. If the argument is the empty list or the empty word, PICK will throw an error.");
//add("Using these commands, you can gather information about logo data structures.");
add("LENGTH returns the length of the given argument. If the argument is a word, LENGTH returns the number of characters it contains. If the argument is a list, LENGTH returns the number of elements it contains.");
add("EMPTYP returns \"TRUE if the argument is the empty word or the empty list, or \"FALSE if it is not.");
add("WORD? returns \"TRUE if the argument is a word, or \"FALSE if it is a list.");
add("LIST? returns \"TRUE if the argument is a list, or \"FALSE if it is a word.");
add("NUMBER? returns \"TRUE if the argument is a number, or \"FALSE if it is a list or a non-numeric word.");
add("MEMBER? returns \"TRUE if the first argument is a member of the second argument, or \"FALSE if it is not. If the second argument is not a list, MEMBER? will throw an error.");
add("IGNORE takes one argument and does nothing with it. This can be useful if a primitive or procedure you must call returns some output that you don't need.");
//add("Using these commands, you can manipulate words as strings.");
add("BEFORE? returns \"TRUE if the first argument comes before the second in lexical order. Case is ignored, so \"apple comes before \"BANANA. If either argument is not a word, BEFORE? throws an error.");
add("ASCII returns the ascii value of the given character as an integer. If the input is a list, or if it is a word whose length is not 1, ASCII will throw an error.");
add("CHAR returns a character whose ascii value is the given integer. If the input is not an integer, CHAR will throw an error.");
add("UPPERCASE returns the given argument with all letters converted to upper case. If the argument is not a word, UPPERCASE throws an error.");
add("LOWERCASE returns the given argument with all letters converted to lower case. If the argument is not a word, LOWERCASE throws an error.");
/**
              !!!!!!!!    !!            !!!!    !!      !!  
              !!          !!          !!    !!  !!      !!  
              !!          !!          !!    !!  !!  !!  !!  
              !!!!!!!!    !!          !!    !!  !!  !!  !!  
              !!          !!          !!    !!    !!  !!    
              !!          !!          !!    !!    !!  !!    
              !!          !!!!!!!!      !!!!      !!  !!    
                                                            
*/
	

add("RUN parses the given list as a sequence of Logo commands, and executes them. If the command list evaluates to a value, RUN returns that value, otherwise it returns nothing. If the argument is not a list, if the list cannot be parsed as a valid sequence of Logo commands, or if an error occurs while running the list, RUN throws an error.");
add("RUNRESULT parses the given list as a sequence of Logo commands, and executes them. If the command list returns a value, RUNRESULT returns a list with that value as the single element, otherwise it returns the empty list. If the argument is not a list, if the list cannot be parsed as a valid sequence of Logo commands, or if an error occurs while running the list, RUNRESULT throws an error.");
//add("APPLY parses the given list as a lambda list, and executes it, creating a local scope and binding the formal arguments with the values given in the argslist. It returns the result of the invocation, or nothing if the lambda returns nothing. If the first argument is a normal lambda list, it executes it as a normal procedure-- that is, you must use the OUTPUT command to return a value, or the STOP command to exit in the middle without returning a value. If the first argument is a macro lambda list, it executes as a macro procedure-- that is, it evaluates directly, and if you call STOP or OUTPUT, it will exit the enclosing procedure. If the first argument is a word, it is interpreted as a procedure name, and that procedure is invoked. The second argument should be a list, whose elements are treated as the arguments to the function invocation. If the second argument is not a list, or the number of elements in the list given does not match the number of formal arguments in the lambda, APPLY throws an error.");
//add("APPLYRESULT parses the first argument as a lambda list, and executes it, creating a local scope and binding the formal arguments with the values given in the argslist. It returns a list containing the result of the invocation as the single element, or the empty list if the lambda returns nothing. If the first argument is a normal lambda list, it executes it as a normal procedure-- that is, you must use the OUTPUT command to return a value, or the STOP command to exit in the middle without returning a value. If the first argument is a macro lambda list, it executes as a macro procedure-- that is, it evaluates directly, and if you call STOP or OUTPUT, it will exit the enclosing procedure. If the first argument is a word, it is interpreted as a procedure name, and that procedure is invoked. The second argument should be a list, whose elements are treated as the arguments to the function invocation. If the second argument is not a list, or the number of elements in the list given does not match the number of formal arguments in the lambda, APPLYRESULT throws an error.");
//add("INVOKE parses the given list as a lambda list, and executes it, creating a local scope and binding the formal arguments with the values given. It returns the result of the invocation, or nothing if the lambda returns nothing. If the first argument is a normal lambda list, it executes it as a normal procedure-- that is, you must use the OUTPUT command to return a value, or the STOP command to exit in the middle without returning a value. If the first argument is a macro lambda list, it executes as a macro procedure-- that is, it evaluates directly, and if you call STOP or OUTPUT, it will exit the enclosing procedure. If the first argument is a word, it is interpreted as a procedure name, and that procedure is invoked. If the number of arguments given does not match the number of formal arguments in the lambda, INVOKE throws an error.");
//add("INVOKERESULT parses the first argument as a lambda list, and executes it, creating a local scope and binding the formal arguments with the values given. It returns a list containing the result of the invocation as the single element, or the empty list if the lambda returns nothing. If the first argument is a normal lambda list, it executes it as a normal procedure-- that is, you must use the OUTPUT command to return a value, or the STOP command to exit in the middle without returning a value. If the first argument is a macro lambda list, it executes as a macro procedure-- that is, it evaluates directly, and if you call STOP or OUTPUT, it will exit the enclosing procedure. If the first argument is a word, it is interpreted as a procedure name, and that procedure is invoked. If the number of arguments given does not match the number of formal arguments in the lambda, INVOKERESULT throws an error.");
//add("MAP parses the first argument as a lambda list, and executes it for each of the arguments given in the argument lists, creating a local scope and binding the formal arguments with the values given. It returns a list containing results of the invocation on each of the elements of the argument lists. If the first argument is a normal lambda list, it executes it as a normal procedure-- that is, you must use the OUTPUT command to return a value, or the STOP command to exit in the middle without returning a value. If the first argument is a macro lambda list, it executes as a macro procedure-- that is, it evaluates directly, and if you call STOP or OUTPUT, it will exit the enclosing procedure. If the first argument is a word, it is interpreted as a procedure name, and that procedure is invoked. If an invocation of the lambda does not return a value, that invocation is ignored during construction of the return list (i.e. the list returned from MAP is shorter.) If the number of argument lists given does not match the number of formal arguments in the lambda, the argument lists are not all of the same length, or the first argument is a procedure name that is not defined, MAP throws an error.");
//add("FOREACH parses the last argument as a lambda list, and executes it for each of the arguments given in the argument lists, creating a local scope and binding the formal arguments with the values given. It does not return anything. If the first argument is a normal lambda list, it executes it as a normal procedure-- that is, you must use the OUTPUT command to return a value, or the STOP command to exit in the middle without returning a value. If the first argument is a macro lambda list, it executes as a macro procedure-- that is, it evaluates directly, and if you call STOP or OUTPUT, it will exit the enclosing procedure. If the first argument is a word, it is interpreted as a procedure name, and that procedure is invoked. If the number of argument lists given does not match the number of formal arguments in the lambda, the argument lists are not all of the same length, the last argument is a procedure name that is not defined, or the function returns a value, FOREACH throws an error.");

add("IF first evaluates the given expression. If it evaluates to \"TRUE, the given list is then parsed as a sequence of Logo commands and executed, otherwise the given list is ignored. IF never returns a value, even if the command list is executed and evaluates to a value. If expr does not evaluate to \"TRUE or \"FALSE, a non-list is given as an input, or if the list cannot be parsed as a valid sequence of Logo commands, or if an error occurs while running the list, IF throws an error.");
add("IFELSE first evaluates the given expression. If it evaluates to \"TRUE, list1 is then parsed as a sequence of Logo commands and executed, otherwise list2 is executed. If the executed list evaluates to a value, then IFELSE returns that value. If expr does not evaluate to \"TRUE or \"FALSE, a non-list is given as an input, or if the list cannot be parsed as a valid sequence of Logo commands, or if an error occurs while running the list, IFELSE throws an error. Note that IF can also behave in the same way as IFELSE by enclosing it in parentheses.");
add("TEST sets the current test value, which is used in subsequent IFTRUE and IFFALSE commands. If the given expression evaluates to \"TRUE then the test value is set to true. If the given expression evaluates to \"FALSE then the test value is set to false. If the given expression does not evaluate to \"TRUE or \"FALSE then TEST throws an error.  If the current test value set by TEST is true, the given list is parsed as a sequence of Logo commands and executed, otherwise the given list is ignored. IFTRUE never returns a value, even if the command list is executed and evaluates to a value. If there is no current test value, a non-list is given as an input, or if the list cannot be parsed as a valid sequence of Logo commands, or if an error occurs while running the list, IFTRUE throws an error.  If the current test value set by TEST is false, the given list is parsed as a sequence of Logo commands and executed, otherwise the given list is ignored. IFFALSE never returns a value, even if the command list is executed and evaluates to a value. If there is no current test value, a non-list is given as an input, or if the list cannot be parsed as a valid sequence of Logo commands, or if an error occurs while running the list, IFFALSE throws an error.");

add("REPEAT The given list is parsed as a sequence of Logo commands and executed the number of times specified by the given integer. REPEAT never returns a value, even if the command list evaluates to a value. If the first input is not a valid integer, a non-list is given as the second input, the list cannot be parsed as a valid sequence of Logo commands, or if an error occurs while running the list, REPEAT throws an error.");
add("REPCOUNT evaluates to the current iteration number in the innermost repeat loop. If invoked while not inside a repeat loop, REPCOUNT returns -1.");
//add("The given condlist is parsed as a sequence of Logo commands. It must evaluate to a boolean value. As long as the condlist evaluates to true, the given cmdlist is parsed as a sequence of Logo commands and repeatedly executed. The cmdlist should not evaluate to a value. The condlist is evaluated first, so it is possible that the cmdlist will never execute. WHILE never returns a value. If either list cannot be parsed as a valid sequence of Logo commands, or if an error occurs while running either list, WHILE throws an error.");
//add("The given condlist is parsed as a sequence of Logo commands. It must evaluate to a boolean value. Until the condlist evaluates to true, the given cmdlist is parsed as a sequence of Logo commands and repeatedly executed. The cmdlist should not evaluate to a value. The condlist is evaluated first, so it is possible that the cmdlist will never execute. UNTIL never returns a value. If either list cannot be parsed as a valid sequence of Logo commands, or if an error occurs while running either list, UNTIL throws an error.");
//add("The given condlist is parsed as a sequence of Logo commands. It must evaluate to a boolean value. As long as the condlist evaluates to true, the given cmdlist is parsed as a sequence of Logo commands and repeatedly executed. The cmdlist should not evaluate to a value. The cmdlist is evaluated first, so it will always execute at least once. DO.WHILE never returns a value. If either list cannot be parsed as a valid sequence of Logo commands, or if an error occurs while running either list, DO.WHILE throws an error.");
//add("The given condlist is parsed as a sequence of Logo commands. It must evaluate to a boolean value. Until the condlist evaluates to true, the given cmdlist is parsed as a sequence of Logo commands and repeatedly executed. The cmdlist should not evaluate to a value. The cmdlist is evaluated first, so it will always execute at least once. DO.UNTIL never returns a value. If either list cannot be parsed as a valid sequence of Logo commands, or if an error occurs while running either list, DO.UNTIL throws an error.");
//add("The given control list should be of the form [varname start end step]. A new symbol level is created on the stack, and the given varname is declared local and set to start. As long as the variable has not gone past end, the given cmdlist is run, and then the variable is changed by step. If there are only three elements in the control list, the step is assumed to be 1. The cmdlist should not evaluate to a value. Going \"past\" end means, more precisely, var&gt;end if step&gt;0, or var&lt;end if step&lt;0. Note that a new symbol level is created within the loop, so any variables declared local within the loop will not exist outside the scope of the loop. FOR never returns a value. If the control list is invalid, the cmdlist list cannot be parsed as a valid sequence of Logo commands, or if an error occurs while running it, FOR throws an error.");


add("GOODBYE exits the Logo runtime environment. This includes shutting down all running threads and closing down any user interface elements. First, the current thread is terminated by propogating the exception to the top of the thread. Once the current thread has been stopped, all other threads are killed. If a GOODBYE is caught and does not propogate to the top of the current thread, the other running threads are left intact and the runtime environment is not terminated.");

/**
  !!!!!!        !!!!    
    !!        !!    !!  
    !!        !!    !!  
    !!        !!    !!  
    !!        !!    !!  
    !!        !!    !!  
  !!!!!!        !!!!    
                        
*/
add("PRINT sends a string representation of the argument to the current writer, followed by an end-of-line. If the argument is a list, the outermost enclosing brackets are not output. If more than one argument is given, PRINT prints them on the same line, separated by spaces. If the current writer is the console, PRINT displays the argument on the console. PRINT also flushes the output buffer. If the current writer cannot be written to, PRINT throws an error.");
add("PRINT1 sends a string representation of the argument to the current writer, but does NOT follow it with an end-of-line. If the argument is a list, the outermost enclosing brackets are not output. If more than one argument is given, PRINT1 prints them on the same line, without any separator. If the current writer is the console, PRINT1 displays the argument on the console. PRINT1 flushes the output buffer if the current writer is the console, but does not flush the output buffer if the current writer is a file or network socket. If the current writer cannot be written to, PRINT1 throws an error.");
add("SHOW sends a string representation of the argument to the current writer, followed by an end-of-line. If the argument is a list, SHOW, unlike PRINT, does output its outermost enclosing brackets. If more than one argument is given, SHOW prints them on the same line, separated by spaces. If the current writer is the console, SHOW displays the argument on the console. SHOW also flushes the output buffer. If the current writer cannot be written to, SHOW throws an error.");
add("SHOW1 sends a string representation of the argument to the current writer, but does NOT follow it with an end-of-line. If the argument is a list, SHOW1, unlike PRINT1, does output its outermost enclosing brackets. If more than one argument is given, SHOW1 prints them on the same line, without any separator. If the current writer is the console, SHOW1 displays the argument on the console. SHOW1 flushes the output buffer if the current writer is the console, but does not flush the output buffer if the current writer is a file or network socket. If the current writer cannot be written to, SHOW1 throws an error.");
add("READLIST reads from the current reader until it encounters an end-of-line, and returns a list containing the line parsed as a list. If the current reader is the console, READLIST blocks until the user has typed a line and pressed the Enter key. If READLIST cannot read from the current reader, or the line cannot be parsed as a list, READLIST throws an error.");
add("READWORD reads from the current reader until it encounters an end-of-line, and returns a word containing the line, not including the end-of-line character. The line is not parsed. If the current reader is the console, READWORD blocks until the user has typed a line and pressed the Enter key. If READWORD cannot read from the current reader, READWORD throws an error.");
add("READCHARACTER reads one byte from the current reader, and returns a word consisting of that byte. If the current reader is the console, READCHARACTER's exact behavior is not defined. Under some consoles, it may not return until the user has typed an entire line and pressed the Enter key. If READCHARACTER cannot read from the current reader, READCHARACTER throws an error.");
//add("PowerMath Logo supports stream-based reading from and writing to files in the local file system and network sockets.");
//add("OPENREAD opens the specified file name for reading only. A stream created using OPENREAD cannot be made the current writer. The file position is initially set at the beginning of the file. The one-input version of OPENREAD returns a unique streamid for the file. The two-input version of OPENREAD returns nothing. If the filename or the streamid is not a word, the specified file does not exist, the file cannot be read, or if local files cannot be opened for security reasons, OPENREAD will throw an error.");
//add("OPENWRITE opens the specified file name for writing only. A stream created using OPENWRITE cannot be made the current reader. If the file already exists, it will first be truncated. The one-input version of OPENWRITE returns a unique streamid for the file. The two-input version of OPENWRITE returns nothing. If the filename or the streamid is not a word, the file exists and cannot be overwritten, or if local files cannot be opened for security reasons, OPENWRITE will throw an error.");
//add("OPENRANDOM opens the specified file name for random-access reading and writing. The one-input version of OPENRANDOM returns a unique streamid for the file. The two-input version of OPENRANDOM returns nothing. If the filename or the streamid is not a word, the file exists and cannot be written to, or if local files cannot be opened for security reasons, OPENRANDOM will throw an error.");
//add("OPENURL opens the specified universal resource locator (url) for reading only. A stream created using OPENURL cannot be made the current writer. The one-input version of OPENURL returns a unique streamid for the file. The two-input version of OPENURL returns nothing. If the url or the streamid is not a word or the specified url cannot be reached over the network, OPENURL will throw an error.");
//add("OPENSOCKET opens a socket to the specified hostname and port for reading and writing. A server must be running on the specified host and port. A stream created using OPENSOCKET can be made a reader, a writer, or both. The two-input version of OPENSOCKET returns a unique streamid for the file. The three-input version of OPENSOCKET returns nothing. If the hostname or the streamid is not a word, the port is not an integer, the specified host and port cannot be reached over the network, or no server is running on the specified host and port, OPENSOCKET will throw an error.");
//add("SERVERSOCKET runs a server on the specified port of the local host. It listens continuously for connection requests. When a connection is made, a stream is created for that connection, and the connectfunc is executed, with the name of the stream as the single argument. Once the function has completed executing, SERVERSOCKET resumes listening. The idlefunc argument is called periodically while SERVERSOCKET is running. idleinterval gives the time in milliseconds between calls to idlefunc. An idleinterval of 0 disables invocation of the idlefunc. idlefunc takes no arguments and should return a boolean value specifying whether to continue running. If idlefunc returns true, SERVERSOCKET continues listening; otherwise it halts. Typically, the connectfunc function should spawn a thread to handle the new connection. Each of the two function arguments may be a lambda list, a procedure name or a primitive name. If the specified port or the specified idleinterval are not integers, the specifed functions are not well-formed or do not specify a defined procedure or primitive, or an error occurs while reparsing either lambda for execution, then SERVERSOCKET returns an error. If an error is thrown out of either function during its execution, then SERVERSOCKET exits and throws that error.");
//add("Several commands are also provided for general stream management, and to manage the current reader and writer.");
//add("SETREAD makes the specified stream the current reader. If the empty list is specified as the streamid, SETREAD sets the Logo console to be the current reader. You can also use the console's streamid, which is \".CONSOLE\". SETREAD does not return a value. If the given streamid is not a word or the empty list, or if it doesn't specify a valid stream, SETREAD throws an error.");
//add("SETWRITE makes the specified stream the current writer. If the empty list is specified as the streamid, SETWRITE sets the Logo console to be the current writer. You can also use the console's streamid, which is \".CONSOLE\". SETWRITE does not return a value. If the given streamid is not a word or the empty list, or if it doesn't specify a valid stream, SETWRITE throws an error.");
//add("READER returns the streamid of the current reader. If the current reader is the console, READER returns the streamid of the console, which is \".CONSOLE\".");
//add("WRITER returns the streamid of the current writer. If the current writer is the console, WRITER returns the streamid of the console, which is \".CONSOLE\".");
//add("CLOSE closes the specified stream. Any further attempts to access a closed stream will result in an error. CLOSE does not return a value. If the given streamid is not a word, if it specifies the console, or if it doesn't specify a valid stream, CLOSE throws an error.");
//add("CLOSEALL closes all open streams except for the console.");
//add("STREAMNAME returns the filename associated with the specified stream. If the stream is a file, STREAMNAME returns its name. If the stream is a URL, STREAMNAME returns the URL. If the stream is a socket, STREAMNAME returns a three-element list consisting of the local port, the remote host, and the remote port. If the streamid is the console, STREAMNAME returns the class name of the console. If the argument is not a word, or does not specify an open stream, then STREAMNAME throws an error.");
//add("STREAMKIND returns the type of the specified stream, which is one of the following values: \"READFILE, \"WRITEFILE, \"SOCKET, \"URL, \"CONSOLE, \"CLOSED, or \"UNKNOWN. If the given streamid does not specify a valid stream, STREAMKIND will return \"CLOSED. If the given streamid is not a word, STREAMKIND throws an error.");
//add("STREAMISRANDOM returns a boolean value denoting whether the specified stream is random-access (that is, whether it supports STREAMPOS, SETSTREAMPOS and STREAMLENGTH). If the given streamid does not specify a valid stream or the stream is closed, STREAMISRANDOM throws an error.");
//add("STREAMPOS returns the current position of the read/write head in the specified stream. If the given streamid does not specify a valid stream, the stream is closed, or the stream does not support random-access operations, STREAMPOS throws an error.");
//add("SETSTREAMPOS sets the current position of the read/write head in the specified stream to the specified value. If the given streamid does not specify a valid stream, the stream is closed, the stream does not support random-access operations, or the second argument is not a positive integer, STREAMPOS throws an error.");
//add("STREAMLENGTH returns the length of the specified stream. If the given streamid does not specify a valid stream, the stream is closed, or the stream does not support random-access operations, STREAMLENGTH throws an error.");
//add("EOF? returns whether the end-of-stream has been reached. If the given streamid specifies the console, EOF? will always return \"FALSE. If the given streamid is not a word, or it does not specify a valid stream, EOF? throws an error.");
//add("KEY? returns whether a character is available on the stream and can be read without blocking. If the given streamid is not a word, or it does not specify a valid stream, KEY? throws an error.");
//add("ALLOPEN returns a list whose members are the streamids of all open streams. The console is not included.");
//add("PowerMath Logo supports a few basic primitives for navigating directories within the file system. For more sophisticated file system access, use the extra plug-in virtuoso.logo.lib.ExtFilePrimitives.");
//add("PWD returns a word containing the path name of the current working directory.");
//add("CWD changes the current working directory to the specified path, which may be an absolute path or a path relative to the current directory. If the path does not specify a valid directory, CWD throws an error.");
//add("CWDUP changes the current working directory to the parent of the current directory. If the current working directory is the root of the file system, CWDUP throws an error. On unix systems, CWDUP is functionally identical to CWD \"..");
/**
            !!      !!      !!      !!!!!!!!!!    !!    !!  
            !!!!  !!!!      !!          !!        !!    !!  
            !!  !!  !!    !!  !!        !!        !!    !!  
            !!  !!  !!    !!  !!        !!        !!!!!!!!  
            !!      !!    !!!!!!        !!        !!    !!  
            !!      !!  !!      !!      !!        !!    !!  
            !!      !!  !!      !!      !!        !!    !!  
                                                            
*/
add("TRUE returns the boolean value true.");
add("FALSE returns the boolean value false.");
add("NOT returns the logical opposite of the argument. If the argument is not a boolean value, NOT throws an error.");
add("ALLOF returns true if all its arguments are true, or false if any of its arguments are false. If any argument is not a boolean value, ALLOF throws an error.");
add("ANYOF returns true if any of its arguments are true, or false if all of its arguments are false. If any argument is not a boolean value, ANYOF throws an error.");
//add("These are the bitwise operations supported by PowerMath Logo.");
//add("BITNOT returns the bitwise not of the argument. If the argument is not an integer, BITNOT throws an error.");
//add("BITAND returns the bitwise and of the arguments. If an argument is not an integer, BITAND throws an error.");
//add("BITOR returns the bitwise or of the arguments. If an argument is not an integer, BITOR throws an error.");
//add("BITXOR returns the bitwise exclusive or of the arguments. If an argument is not an integer, BITXOR throws an error.");
//add("LSHIFT shifts the bits of the first number left by the second number. To shift right, pass a negative number as the second argument. 0's shifted into all vacated digits. If an argument is not an integer, LSHIFT throws an error.");
//add("ASHIFT shifts the bits of the first number left by the second number. To shift right, pass a negative number as the second argument. ASHIFT preserves the sign of the number. That is, if the number is being shifted to the right, and the high-order bit of the original number is a 1, then 1's are shifted into the left end. If the high-order bit of the original number is a 0, then 0's are shifted into the left end. If an argument is not an integer, ASHIFT throws an error.");
//add("These are the numeric comparison operations supported by PowerMath Logo.");
add("EQUAL? returns true if and only if the arguments are equal. If both arguments are numbers, EQUAL? returns numeric equality, otherwise, it returns string equality or list equality.");
add("GREATER? returns true if and only if expr is numerically greater than expr2. If an argument is not a number, GREATER? throws an error.");
add("LESS? returns true if and only if expr is numerically less than expr2. If an argument is not a number, LESS? throws an error.");
add("GREATEREQUAL? returns true if and only if expr is numerically greater than or equal to expr2. If an argument is not a GREATEREQUAL, GREATER? throws an error.");
add("LESSEQUAL? returns true if and only if expr is numerically less than or equal to expr2. If an argument is not a number, LESSEQUAL? throws an error.");
//add("PowerMath Logo includes a full set of arithmetic functions.");
add("INTEGER returns the integer portion of the argument. If the argument is not a number, INTEGER throws an error.");
add("ROUND rounds the argument to the nearest integer. If the argument is not a number, ROUND throws an error.");
add("ABS returns the absolute value of the argument. If the argument is not a number, ABS throws an error.");
add("MINUS returns the negation of the argument. If the argument is not a number, MINUS throws an error. Note that there is a precedence difference between MINUS and -.");
add("SUM returns the sum of the arguments. If an argument is not a number, SUM throws an error.");
add("DIFFERENCE returns the difference of the arguments. If an argument is not a number, DIFFERENCE throws an error.");
add("PRODUCT returns the product of the arguments. If an argument is not a number, PRODUCT throws an error.");
add("QUOTIENT divides expr1 by expr2 and returns the quotient. The quotient is a floating-point number if expr1 is not evenly divisible by expr2. If an argument is not a number, or if expr2 is 0, QUOTIENT throws an error.");
add("REMAINDER divides expr1 by expr2 using integer division. The remainder is returned. The quotient is ignored. If an argument is a non-integer value, it is first rounded to the nearest integer. If an argument is not a number, or if expr2 is 0, REMAINDER throws an error.");
add("SQRT returns the square root of the argument. If the argument is not a number, or if it is a negative number, SQRT throws an error.");
add("POWER returns expr1 raised to the power of expr2. If an argument is not a number, or both arguments are zero, or expr1 is negative and expr2 is not an integer, POWER throws an error.");
add("EXP returns the constant <i>e</i> raised to the power of the argument. If the argument is not a number, EXP throws an error.");
add("LOG returns the natrual logarithm (log to the base <i>e</i>) of the argument. If the argument is not a number, or if it is a non-positive number, LOG throws an error.");
add("RANDOM returns a pseudo-random integer between 0 and the given argument minus 1, inclusive. If the argument is not a positive integer, RANDOM throws an error.");
add("RANDOMIZE sets the seed for sequence of pseudo-random numbers to be returned by subsequent calls to RANDOM. Every time PowerMath Logo is started up, the seed is set to 0; therefore, if you restart PowerMath Logo, the same sequence of numbers will be generated. Use RANDOMIZE to set a seed at \"random\" depending on the value of the system time stamp. The version of RANDOMIZE with an argument uses the argument as a seed. If the argument is not an integer, RANDOMIZE throws an error.");
//add("PowerMath Logo includes a full set of trigonometric functions.");
add("SIN returns the sine of the argument (given in degrees). If the argument is not a number, SIN throws an error.");
add("COS returns the cosine of the argument (given in degrees). If the argument is not a number, COS throws an error.");
add("TAN returns the tangent of the argument (given in degrees). If the argument is not a number, or if the input value is too close to an odd multiple of 90, TAN throws an error.");
add("RADSIN returns the sine of the argument (given in radians). If the argument is not a number, RADSIN throws an error.");
add("RADCOS returns the cosine of the argument (given in radians). If the argument is not a number, RADCOS throws an error.");
add("RADTAN returns the tangent of the argument (given in radians). If the argument is not a number, or if the argument is too close to an odd multiple of pi/2, RADTAN throws an error.");
add("ARCSIN returns the inverse sine of the argument, in degrees. The returned value will be between -90 and 90. If the argument is not a number, or if it is greater than 1.0 or less than -1.0, ARCSIN throws an error.");
add("ARCCOS returns the inverse cosine of the argument, in degrees. The returned value will be between 0 and 180. If the argument is not a number, or if it is greater than 1.0 or less than -1.0, ARCCOS throws an error.");
add("ARCTAN returns the inverse tangent of the argument, in degrees. The returned value will be between -90 and 90. If the argument is not a number, ARCTAN throws an error.");
add("ARCTAN2 returns the inverse tangent of expry/exprx, in degrees, by finding the angle formed by the points (x,y), the origin, and (1,0). The returned value will be between -180 and 180. ARCTAN2 0 0 is defined as 0. If any of the arguments is not a number, ARCTAN2 throws an error.");
add("RADARCSIN returns the inverse sine of the argument, in radians. The returned value will be between -pi/2 and pi/2. If the argument is not a number, or if it is greater than 1.0 or less than -1.0, RADARCSIN throws an error.");
add("RADARCCOS returns the inverse cosine of the argument, in radians. The returned value will be between 0 and pi. If the argument is not a number, or if it is greater than 1.0 or less than -1.0, RADARCCOS throws an error.");
add("RADARCTAN returns the inverse tangent of the argument, in radians. The returned value will be between -pi/2 and pi/2. If the argument is not a number, RADARCTAN throws an error.");
add("RADARCTAN2 returns the inverse tangent of expry/exprx, in radians, by finding the angle formed by the points (x,y), the origin, and (1,0). The returned value will be between -pi and pi. RADARCTAN2 0 0 is defined to be 0. If any of the arguments is not a number, RADARCTAN2 throws an error.");
add("PI returns an approximation of the value pi. In PowerMath Logo, this value is equal to 3.14159265358979323846.");

/**
            !!      !!      !!!!      !!!!!!      !!      !!
            !!      !!    !!    !!    !!    !!    !!    !!  
            !!  !!  !!    !!    !!    !!    !!    !!  !!    
            !!  !!  !!    !!    !!    !!!!!!      !!!!!!    
              !!  !!      !!    !!    !!  !!      !!    !!  
              !!  !!      !!    !!    !!    !!    !!    !!  
              !!  !!        !!!!      !!      !!  !!      !!
                                                            
*/
/**
                !!!!          !!                            
                  !!                                        
  !!!!!!          !!        !!!!      !!!!!!        !!!!!!  
        !!        !!          !!            !!    !!        
    !!!!!!        !!          !!        !!!!!!      !!!!    
  !!    !!        !!          !!      !!    !!          !!  
    !!!!!!        !!!!        !!        !!!!!!    !!!!!!    
                                                            
*/
		ABREVIATE("EQUAL?", "=" );
		ABREVIATE("GREATER?",">");
		ABREVIATE("LESS?","<");
		ABREVIATE("GREATEREQUAL?",">=");
		ABREVIATE("GREATEREQUAL?","GREATEREQUALP");
		ABREVIATE("LESSEQUAL?","<=");
		ABREVIATE("LESSEQUAL?","LESSEQUALP");
		ABREVIATE("SUM","+");
		ABREVIATE("DIFFERENCE", "-");
	
		ABREVIATE( "BUTFIRST", "BF" );
		ABREVIATE( "EOF?", "EOFP" );
		ABREVIATE( "EQUAL?", "EQUALP" );
		ABREVIATE( "ERASENAME", "ERN" );
		ABREVIATE( "ERASEPROCEDURE", "ERP" );
		ABREVIATE( "ERASEPLIST", "ERPL" );
		ABREVIATE( "INTEGER", "INT" );
		ABREVIATE( "KEY?", "KEYP" );
		ABREVIATE( "LENGTH","COUNT");
		ABREVIATE( "LESSEQUAL?", "LESSEQUALP");
		ABREVIATE( "LESS?", "LESSP" );
		ABREVIATE( "LIST?","LISTP");
		ABREVIATE("MACRO?","MACROP");
		ABREVIATE("NAME?","NAMEP");
		ABREVIATE("NUMBER?","NUMBERP");
		ABREVIATE("PRINTOUT","PO");
		ABREVIATE("PRINTOUTNAME","PON");
		ABREVIATE("PRINTOUTPROCEDURE","POP");
		ABREVIATE("PRINTOUTPLIST","POPL");
		ABREVIATE("POWER", "^" );
		ABREVIATE("PRINT","PR");
		ABREVIATE("PRIMITIVE?","PRIMITIVEP");
		ABREVIATE("PRINT1","TYPE");
		ABREVIATE("PROCEDURE?","PROCEDUREP" );
		ABREVIATE("PRODUCT", "*" );
		ABREVIATE("QUOTIENT","/" );
		ABREVIATE("RANDOMIZE","RERANDOM");
		ABREVIATE("SENTENCE","SE");
		ABREVIATE("SUM","+");
		ABREVIATE("WORD?","WORDP");
		// TURTLE PRIMITIVES 
		ABREVIATE("BACK","BK");
		ABREVIATE("CLEARSCREEN","CS");
		ABREVIATE("CLEARGRAPHICS","CG");
		ABREVIATE("CLEARTEXT","CT" );
		ABREVIATE("FORWARD", "FD");
		ABREVIATE("GETBACKGROUND","GETBG");
		ABREVIATE("GETPENCOLOR","GETPC");
		ABREVIATE("HIDETURTLE", "HD" );
		ABREVIATE("LEFT", "LT");
		ABREVIATE("PALETTE?", "PALETTEP");
		ABREVIATE("PENDOWN", "PD");
		ABREVIATE("PENERASE","PE");
		ABREVIATE("PENUP", "PU" );
		ABREVIATE("RIGHT","RT");
		ABREVIATE("SETBACKGROUND","SETBG");
		ABREVIATE("SETHEADING","SETH");
		ABREVIATE("SETPENCOLOR","SETPC");
		ABREVIATE("SETPENCOLOR","SETC");
		ABREVIATE("SHOWTURTLE","ST");
		ABREVIATE("XCOR","GETX");
		ABREVIATE("YCOR","GETY");
		ABREVIATE("GETFONTCOLOR","GETFC");
		ABREVIATE("SETFONTCOLOR","SETFC");
		ABREVIATE("GETFONTSIZE","GETFS");
		ABREVIATE("SETFONTSIZE","SETFS");
		ABREVIATE("SETSHAPE","SETSH");
		
		
		// FROM ZZ PRIMITIES
		ABREVIATE("ISFRACTION","ISFRAC");
		ABREVIATE("FRACTIONNUMERATOR","NUMERATOR");
		ABREVIATE("FRACTIONNUMERATOR","FRACNU" );
		ABREVIATE("FRACTIONNUMERATOR","FRACN");
		ABREVIATE("FRACTIONDENOMINATOR","DENOMINATOR");
		ABREVIATE("FRACTIONDENOMINATOR","FRACDE");
		ABREVIATE("FRACTIONDENOMINATOR","FRACD");
		ABREVIATE("MEMBER?","MEMBERP");
		ABREVIATE("READCHARACTER","RC");
		ABREVIATE("READLIST","RL");
		ABREVIATE("READWORD","RW");
		
		
	}

}