
/* ***************************************************
	FILE:  TurtleDisplayThread.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.logo.lib;


// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

/**
 * Redisplay thread inner class
 */
public final class TurtleDisplayThread
extends Thread
{
	private boolean _toggle;
	private boolean _alive;
	private int _interval;
	
	public TurtleDisplayThread() {
		_toggle = false;
		_alive = true;
		_interval = 30;
	}
	
	public void setTurtlePanel( final TurtlePanel turtlePanel ) {
		_turtlePanel = turtlePanel;
	}
	private TurtlePanel _turtlePanel;
	
	/**
	 * Toggle the thread
	 */
	synchronized final void toggle() {
		_toggle = true;
	}
	/**
	 * Kill the thread
	 */
	synchronized final void kill() {
		_alive = false;
		notifyAll();
	}
	/**
	 * Set the thread's tick interval
	 */
	public synchronized final void setInterval( int val) {
		_interval = val;
		notifyAll();
	}
	/**
	 * Run the thread
	 */
	public void run() {
		boolean update = false;
		
		while (_alive) {
			synchronized(this) {
				try {
					if (_interval > 0) {
						wait(_interval);
					}
					else {
						wait();
					}
				}
				catch (InterruptedException e) {
				}
				update = (_alive && _toggle);
				_toggle = false;
			}
			if (update) {
				updateWind();
			}
		}
	}
	/**
	 * Update graph window
	 */
	private synchronized final void updateWind() {
		if (_turtlePanel != null) {
			_turtlePanel.repaint();
		}
	}
}