
/* ***************************************************
	FILE:  TurtlePanel.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.logo.lib;

// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.*;
/**
	Graphics window for the turtle
	
	Mon Jul 11 16:10:29 CDT 2005,
		pulled out of the TurtlePrimitives class
*/
public final class TurtlePanel extends JTextArea
{
	static final int TURTLE_HEIGHT = 16;
	static final int TURTLE_HALF_WIDTH = 6;
	
	static final Color TURTLE_COLOR = Color.black;
	
	public void setBackColor( final Color color ) {
		_backColor = color;
		
		// ==========================================
		// Fri Jul 29 00:18:18 CDT 2005
		//   added this since setbg should immediately
		//   update the graphics, before it would need
		//   another call like CLEARSCREEN, which seems
		//   to not be user expected behavior
		_graphics.setColor(_backColor);
		_graphics.fillRect(0, 0, _xsize, _ysize);
		repaint();
		// ==========================================
		
	}
	public Color getBackColor() { return _backColor; }
	private Color _backColor = Color.white;
	
	public void setPenColor( final Color penColor ) {
		_penColor = penColor;
	}
	private Color _penColor;
	public Color getPenColor() { return _penColor; }
	
	Graphics _graphics;
	
	//:RedisplayInterval
		public final int getRedisplayInterval(){return _RedisplayInterval;}
		public final void setRedisplayInterval(final int aRedisplayInterval){
			
			_RedisplayInterval=aRedisplayInterval;
			
			
		}
		private int _RedisplayInterval;
	
	public Image getOffscreenImage() { return _offImage; }
	private Image _offImage;
	private int _xsize;
	private int _ysize;
	
	public Image getOffscreenImageWithText() {
		Image offImage = createImage(_xsize, _ysize);
		Graphics g = offImage.getGraphics();
		//g.drawImage(_offImage, 0, 0, this);
		
		paint( g );
		
		return offImage;
	}
	
	
	
	private double _turtleX;
	private double _turtleY;
	private double _turtleHead;
	private boolean _turtleVis;
	private boolean _turtleWrap;

	private int _xhalfsize;
	private int _yhalfsize;
	
	public void setTurtleDisplayThread( final TurtleDisplayThread t ) {
		_thread = t;
	}
	private TurtleDisplayThread _thread;
	
	public TurtlePanel( final int xhalfsize, final int yhalfsize ) throws LanguageException {
		_xhalfsize = xhalfsize;
		_yhalfsize = yhalfsize;
		//setLayout(new BorderLayout());
		//setResizable(false);
		//setTitle("Logo Graphics");
		_xsize = _xhalfsize*2;
		_ysize = _yhalfsize*2;
		//setSize(_xsize, _ysize);
		//show();
		setOpaque( false );
		setLineWrap( true );
		setFont( UIAid.SMALL_FONT_BOLD );
	}
	public void init() throws LanguageException {
		_offImage = createImage(_xsize, _ysize);
		if (_offImage == null) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePanel.java.ux:TL-167"+" "+"Unable to open graphics window");
				}
				else {
					throw new LanguageException("Unable to open graphics window");
				}
		}
		_graphics = _offImage.getGraphics();
		_graphics.setColor(_backColor);
		_graphics.fillRect(0, 0, _xsize, _ysize);
		_graphics.setFont(Font.decode("Monospaced"));
	}
	/**
	 * Reset a TurtlePanel
	 */
	final void reset() {
		if (_xsize != _xhalfsize*2 || _ysize != _yhalfsize*2) {
			_offImage = createImage(_xhalfsize*2, _yhalfsize*2);
			_xsize = _xhalfsize*2;
			_ysize = _yhalfsize*2;
			//setResizable(true);
			//setSize(_xsize, _ysize);
			//setResizable(false);
			_graphics = _offImage.getGraphics();
		}
		_graphics.setColor(_backColor);
		_graphics.fillRect(0, 0, _xsize, _ysize);
		
		repaint();
	}
	/**
	 * Redraw the window
	 */
	public void paint( Graphics g) {
		try{
			g.drawImage(_offImage, 0, 0, this);
			if (_turtleVis) {
				g.setColor(TURTLE_COLOR);
				drawTurtle(g, _turtleX, _turtleY);
				if (_turtleWrap) {
					if (_turtleX > _xhalfsize-20) {
						drawTurtle(g, _turtleX-_xsize, _turtleY);
						if (_turtleY > _yhalfsize-20) {
							drawTurtle(g, _turtleX, _turtleY-_ysize);
							drawTurtle(g, _turtleX-_xsize, _turtleY-_ysize);
						}
						else if (_turtleY < -_yhalfsize+20) {
							drawTurtle(g, _turtleX, _turtleY+_ysize);
							drawTurtle(g, _turtleX-_xsize, _turtleY+_ysize);
						}
					}
					else if (_turtleX < -_xhalfsize+20) {
						drawTurtle(g, _turtleX+_xsize, _turtleY);
						if (_turtleY > _yhalfsize-20) {
							drawTurtle(g, _turtleX, _turtleY-_ysize);
							drawTurtle(g, _turtleX+_xsize, _turtleY-_ysize);
						}
						else if (_turtleY < -_yhalfsize+20) {
							drawTurtle(g, _turtleX, _turtleY+_ysize);
							drawTurtle(g, _turtleX+_xsize, _turtleY+_ysize);
						}
					}
					else {
						if (_turtleY > _yhalfsize-20) {
							drawTurtle(g, _turtleX, _turtleY-_ysize);
						}
						else if (_turtleY < -_yhalfsize+20) {
							drawTurtle(g, _turtleX, _turtleY+_ysize);
						}
					}
				}
			}
			super.paint(g);
		}
		catch(Exception e) {
			// don't do anything!!!
		}
	}
	/**
	 * Draw the turtle
		// SHOW_IMAGE 100 "app-images" "images/turtletracksimages/turtles.jpg"
		The turtle consists of the circle centered at the turtle location and its four legs and neck along with a triangle for the head.
	 */
	final void drawTurtle( Graphics g, double x, double y) {
	
		final int turtleSize = 5;
		final int neckLength = 4;
		final int legLength = 2;
		// the legs are this angle from the center line (neck) in radians
		final double legAngle = (Math.PI)/4.0;
		
		final int tx = _xhalfsize+(int)x;
		final int ty = _yhalfsize-(int)y;
		
		final double cos = Math.cos(_turtleHead);
		final double sin = Math.sin(_turtleHead);
		
		
		// DRAW THE NECK
		int nx1 = (int)(tx+(turtleSize)*cos);
		int ny1 = (int)(ty-(turtleSize)*sin);
		int nx2 = (int)(tx+(turtleSize+neckLength)*cos);
		int ny2 = (int)(ty-(turtleSize+neckLength)*sin);
		g.drawLine( nx1, ny1, nx2, ny2 );
		
		// DRAW THE FRONT RIGHT LEG
		drawLeg(g,tx,ty,turtleSize,legLength,_turtleHead-legAngle);
		// DRAW THE FRONT LEFT LEG
		drawLeg(g,tx,ty,turtleSize,legLength,_turtleHead+legAngle);
		// DRAW THE BACK RIGHT LEG
		drawLeg(g,tx,ty,turtleSize,legLength,_turtleHead+legAngle+Math.PI);
		// DRAW THE BACK LEFT LEG
		drawLeg(g,tx,ty,turtleSize,legLength,_turtleHead-legAngle-Math.PI);
		
		//System.out.println("angle is " + _turtleHead + " sin : " + sin + " cos " + cos+"<:|:>"+"virtuoso.logo.lib.TurtlePanel.java.ux:TL-280");
		
		// DRAW THE ROUND BODY
		g.fillOval( tx-turtleSize, ty-turtleSize,turtleSize*2,turtleSize*2 );
		
		// DRAW THE HEAD
		final int HEAD_SIZE = 2;
		final int hx = nx2;
		final int hy = ny2;
		g.fillOval( hx-HEAD_SIZE, hy-HEAD_SIZE,HEAD_SIZE*2,HEAD_SIZE*2 );
		
		/*
		int x1 = _xhalfsize+(int)(Math.round(x+TURTLE_HEIGHT*cos));
		int y1 = _yhalfsize-(int)(Math.round(y+TURTLE_HEIGHT*sin));
		int x2 = _xhalfsize+(int)(Math.round(x+TURTLE_HALF_WIDTH*sin));
		int y2 = _yhalfsize-(int)(Math.round(y-TURTLE_HALF_WIDTH*cos));
		int x3 = _xhalfsize+(int)(Math.round(x-TURTLE_HALF_WIDTH*sin));
		int y3 = _yhalfsize-(int)(Math.round(y+TURTLE_HALF_WIDTH*cos));
		g.drawLine(x1, y1, x2, y2);
		g.drawLine(x2, y2, x3, y3);
		g.drawLine(x3, y3, x1, y1);
		
		*/
		
		
		drawTurtleAsImage( (Graphics2D)g, UIAid.TURTLE_IMAGE, tx, ty, _turtleHead );
	}
	public void drawTurtleAsImage( final Graphics2D g2d, final Image image, int x, int y, final double radians ) {
	
		//x = /*getClipX() + */ getClipWidth() / 2;
		//y = /*getClipY() + */ getClipHeight() / 2;
		
		final double sx = .015;
		final double sy = .015;
	
		int w = image.getWidth( this );
		int h = image.getHeight( this );
		
		final AffineTransform transform = new AffineTransform(); 
		//transform.scale( .025, .025 );
		//transform.translate( x , y  );
		transform.translate( x - w/2*sx, y - h/2*sy );
		
		// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
		// ROTATE THE IMAGE
		transform.scale( sx, sy );
		transform.translate( w/2, h/2 );
		transform.rotate(-radians - (Math.PI/2) );
		transform.translate( -w/2, -h/2);
		// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
		
		//transform.translate( x, y );
		final AffineTransform translate = new AffineTransform();
		
		g2d.drawImage( image, transform, this ); 
		
	}
	
	private final void drawLeg( final Graphics g, final int tx, final int ty, final int turtleSize, final int legLength, final double legAngle ) {
		final int nx1 = (int)(tx+(turtleSize)*Math.cos( legAngle ) );
		final int ny1 = (int)(ty-(turtleSize)*Math.sin( legAngle ) );
		final int nx2 = (int)(tx+(turtleSize+legLength)*Math.cos( legAngle ) );
		final int ny2 = (int)(ty-(turtleSize+legLength)*Math.sin( legAngle ) );
		g.drawLine( nx1, ny1, nx2, ny2 );
	}
	/**
	 * Set turtle state
	 x and y are in screen coordinates
	 h is in standard polar coordinates
	 */
	final void setTurtleState( double x, double y, double h, boolean v, boolean w ) {
		_turtleX = x;
		_turtleY = y;
		_turtleHead = h;
		_turtleVis = v;
		_turtleWrap = w;
		redisplay();
	}
	/**
	 * Redisplay screen
	 */
	final void redisplay() {
		final int redisplayInterval = getRedisplayInterval();
		if (redisplayInterval == 0) {
			repaint();
		}
		else if (redisplayInterval > 0) {
			_thread.toggle();
		}
	}
}