
/* ***************************************************
	FILE:  TurtlePrimitives.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
// 2D turtle graphics primitives
package virtuoso.logo.lib;


// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

/**
 * Names of primitives (keywords) _redisplayInterval
 */
public class TurtlePrimitives
extends PrimitiveGroup
{
	
	private byte _penState;
	private double _xpos;
	private double _ypos;
	int _xhalfsize;
	int _yhalfsize;
	private double _heading;
	private boolean _turtleVisible;
	private byte _wrapState;
	
	private TurtlePanel getTurtlePanel() {
		return _turtlePanel;
	}
	public void setTurtlePanel( final TurtlePanel tp ) {
		_turtlePanel = tp;
		readyTheTurtle(_turtlePanel.getSize().width / 2, _turtlePanel.getSize().height / 2);
	}
	private TurtlePanel _turtlePanel;
	public void setTurtleDisplayThread( final TurtleDisplayThread thread ) {
		_turtleDisplayThread = thread;
	}
	private TurtleDisplayThread _turtleDisplayThread;
	//private TurtlePanel _turtlePanel;
	//private Frame _turtleFrame;
	
	private Frame makeTurtleFrame( final TurtlePanel p, final int w, final int h ) throws LanguageException {
		Frame f = new Frame();
		f.setLayout( new BorderLayout() );
		f.setResizable( false );
		f.setTitle( "Logo Graphics" );
		f.setSize( w, h );
		f.add( p, BorderLayout.CENTER );
		f.show();
		
		p.init();
		return f;
	}
	
	private Hashtable _colorNames;
	private Machine _mach;
	//private TurtleDisplayThread _turtleDisplayThread;
	
	
	private static final byte PS_DOWN = 1;
	private static final byte PS_UP = 2;
	private static final byte PS_ERASE = 3;
	private static final byte WS_WRAP = 1;
	private static final byte WS_FENCE = 2;
	private static final byte WS_WINDOW = 3;
	
	/*
	//:There is a throttle for the speed of doing the commands that move the turtle, these commands are:
	//: pLEFT, pRIGHT, pFORWARD, pBACK
	The slider will have values from 0 to 100.
	0 means maximum slowdown
	100 means no slow down
	
	*/
	private final int MIN_THROTTLE = 1;
	private final int MAX_THROTTLE = 100;
	private final long MAX_DELAY = 50;
	private final long STEPS_PER_LOOP = 1000;
	
	public final BoundedRangeModel getThrottleBoundedRangeModel(){return _rangeModel;}
	private final BoundedRangeModel _rangeModel = new DefaultBoundedRangeModel(95,0,MIN_THROTTLE,MAX_THROTTLE);
	private final int getThrottle(){
		return MAX_THROTTLE - getThrottleBoundedRangeModel().getValue();
	}
	
	
	public long getThrottleDelay() {
		final int t = getThrottle();
		final float ratio = ((float)t) / ((float)MAX_THROTTLE);
		return (long)(MAX_DELAY * ratio);
	}
	
	/**
	 * Set up primitive group
	 */
	protected void setup(Machine mach,virtuoso.logo.Console console) throws SetupException {
		registerPrimitive("BACK", "pBACK", 1);
		registerPrimitive("BK", "pBACK", 1);
		registerPrimitive("FD", "pFORWARD", 1);
		registerPrimitive("FORWARD", "pFORWARD", 1);
		
		registerPrimitive("CLEAN", "pCLEAN", 0); // clean leaves turtle
		
		registerPrimitive("CLEARSCREEN", "pCLEARSCREEN", 0);
		registerPrimitive("CS", "pCLEARSCREEN", 0);
		registerPrimitive("CLEARGRAPHICS", "pCLEARGRAPHICS", 0);
		registerPrimitive("CG", "pCLEARGRAPHICS", 0);
		registerPrimitive("CLEARTEXT", "pCLEARTEXT", 0);
		registerPrimitive("CT", "pCLEARTEXT", 0);
		
		registerPrimitive("DISTANCETO", "pDISTANCETO", 1);
		registerPrimitive("DISTANCETOXY", "pDISTANCETOXY", 2);
		//registerPrimitive("DRAW", "pDRAW", 0);
		// ======================================
		// Sun Jul 31 18:27:06 CDT 2005
		// Fri Feb 03 20:08:24 CST 2006, changed to accept two lists for points
		registerPrimitive( "FILL", "pFILL", 2 );
		// ======================================
		registerPrimitive("FENCE", "pFENCE", 0);
		
		registerPrimitive("GETBACKGROUND", "pGETBACKGROUND", 0);
		registerPrimitive("GETBG", "pGETBACKGROUND", 0);
		registerPrimitive("GETPC", "pGETPENCOLOR", 0);
		registerPrimitive("GETPENCOLOR", "pGETPENCOLOR", 0);
		registerPrimitive("HEADING", "pHEADING", 0);
		registerPrimitive("HIDETURTLE", "pHIDETURTLE", 0);
		registerPrimitive("HOME", "pHOME", 0);
		registerPrimitive("HT", "pHIDETURTLE", 0);
		registerPrimitive("LABEL", "pLABEL", 1);
		registerPrimitive("LEFT", "pLEFT", 1);
		registerPrimitive("LT", "pLEFT", 1);
		//registerPrimitive("ND", "pNODRAW", 0);
		//registerPrimitive("NODRAW", "pNODRAW", 0);
		registerPrimitive("PALETTE", "pPALETTE", 1);
		registerPrimitive("PALETTE?", "pPALETTEP", 1);
		registerPrimitive("PALETTEP", "pPALETTEP", 1);
		registerPrimitive("PD", "pPENDOWN", 0);
		registerPrimitive("PE", "pPENERASE", 0);
		registerPrimitive("PENDOWN", "pPENDOWN", 0);
		registerPrimitive("PENERASE", "pPENERASE", 0);
		registerPrimitive("PENUP", "pPENUP", 0);
		registerPrimitive("POS", "pPOS", 0);
		registerPrimitive("PU", "pPENUP", 0);
		registerPrimitive("REFRESH", "pREFRESH", 0);
		registerPrimitive("REFRESHINTERVAL", "pREFRESHINTERVAL", 1);
		registerPrimitive("RESETPALETTE", "pRESETPALETTE", 1);
		registerPrimitive("RIGHT", "pRIGHT", 1);
		registerPrimitive("RT", "pRIGHT", 1);
		registerPrimitive("SETBACKGROUND", "pSETBACKGROUND", 1);
		registerPrimitive("SETBG", "pSETBACKGROUND", 1);
		registerPrimitive("SETH", "pSETHEADING", 1);
		registerPrimitive("SETHEADING", "pSETHEADING", 1);
		registerPrimitive("SETPALETTE", "pSETPALETTE", 1);
		registerPrimitive("SETPC", "pSETPENCOLOR", 1);
		registerPrimitive("SETC", "pSETPENCOLOR", 1);//added Sun Jul 31 03:33:14 CDT 2005
		registerPrimitive("SETPENCOLOR", "pSETPENCOLOR", 1);
		registerPrimitive("SETPOS", "pSETPOS", 1);
		registerPrimitive("SETX", "pSETX", 1);
		registerPrimitive("SETXY", "pSETXY", 2);
		registerPrimitive("SETY", "pSETY", 1);
		registerPrimitive("SHOWTURTLE", "pSHOWTURTLE", 0);
		registerPrimitive("ST", "pSHOWTURTLE", 0);
		registerPrimitive("TOWARDS", "pTOWARDS", 1);
		registerPrimitive("TOWARDSXY", "pTOWARDSXY", 2);
		registerPrimitive("UNSETPALETTE", "pUNSETPALETTE", 1);
		registerPrimitive("WINDOW", "pWINDOW", 0);
		registerPrimitive("WRAP", "pWRAP", 0);
		registerPrimitive("XCOR", "pXCOR", 0);
		registerPrimitive("GETX", "pXCOR", 0);
		registerPrimitive("XSIZE", "pXSIZE", 0);
		registerPrimitive("YCOR", "pYCOR", 0);
		registerPrimitive("GETY", "pYCOR", 0);
		registerPrimitive("YSIZE", "pYSIZE", 0);
		
		registerPrimitive("TILES", "pTILES", 0);
		
		registerPrimitive("GETFC", "pGETFONTCOLOR", 0);
		registerPrimitive("GETFONTCOLOR", "pGETFONTCOLOR", 0);
		registerPrimitive("SETFONTCOLOR", "pSETFONTCOLOR", 1);
		registerPrimitive("SETFC", "pSETFONTCOLOR", 1);
		
		registerPrimitive("GETFS", "pGETFONTSIZE", 0);
		registerPrimitive("GETFONTSIZE", "pGETFONTSIZE", 0);
		registerPrimitive("SETFONTSIZE", "pSETFONTSIZE", 1);
		registerPrimitive("SETFS", "pSETFONTSIZE", 1);
		
		registerPrimitive("SETSH","pSETSHAPE",1);
		registerPrimitive("SETSHAPE","pSETSHAPE",1);
		registerPrimitive("STAMP","pSTAMP",0);
		
		_mach = mach;
		//_turtlePanel = null;
		//_turtleDisplayThread = null;
		resetPalette();
		
		console.putStatusMessage("PowerMath Logo turtle graphics primitives v1.0");
		
		
	}
	
	/**
	 * VM is closing down
	 */
	protected void exiting()
	{
		
//		synchronized (this)
//		{
//			if (_turtleFrame != null) {
//				_turtleFrame.dispose();
//				_turtleFrame = null;
//				_turtlePanel = null;
//				_turtleDisplayThread.kill();
//				_turtleDisplayThread = null;
//			}
//		}
	}
	/**
	 * Primitive implementation for BACK
	 */
	public final LogoObject pBACK( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		double dist = -params[0].toNumber();
		moveAmount(dist);
		return LogoVoid.obj;
	}

	/**
		Needed for the throttling ability.
	*/
	private final void moveAmount(double dist) throws LanguageException {
		double headingRad = (90.0-_heading)/180.0*Math.PI;
		if( getThrottleDelay() == 0 ) {
			synchronized (this) {
				//double headingRad = (90.0-_heading)/180.0*Math.PI;
				moveTo(_xpos+dist*Math.cos(headingRad), _ypos+dist*Math.sin(headingRad));
			}
		} else {
			final double ONE_THOUSANDTH = ((dist >= 0) ? .001D : -.001D);
			dist = Math.abs(dist);
			double inc = Math.abs( ONE_THOUSANDTH );
			int sleepy = 0;
			for( double i = 0; i < dist; i += inc ) {
				synchronized (this) {
					//double headingRad = (90.0-_heading)/180.0*Math.PI;
					moveTo(_xpos+ONE_THOUSANDTH*Math.cos(headingRad), _ypos+ONE_THOUSANDTH*Math.sin(headingRad));
				}
				
	sleepy++;
	if( sleepy==STEPS_PER_LOOP ) {
		sleepy = 0;
		try {
			Thread.currentThread().sleep(getThrottleDelay());// IN CASE USER IS SPEEDING UP AS THIS LOOP GOES
		} catch(Exception e) {
			//System.out.println("exception: " + e);
			break;
		}
	}
			}
		}
	}
	private final void turnAmount(double val) throws LanguageException {
		if( getThrottleDelay() == 0 ) {
			synchronized (this) {
				_heading = fixAngle( _heading + val );
				updateTurtle();
			}
		} else {
			final double ONE_THOUSANDTH = ((val >= 0) ? .001D : -.001D);
			val = Math.abs(val);
			double inc = Math.abs( ONE_THOUSANDTH );
			int sleepy = 0;
			for( double i = 0; i < val; i += inc ) {
				synchronized (this) {
					_heading = fixAngle( _heading + ONE_THOUSANDTH );
					updateTurtle();
				}
				
	sleepy++;
	if( sleepy==STEPS_PER_LOOP ) {
		sleepy = 0;
		try {
			Thread.currentThread().sleep(getThrottleDelay());// IN CASE USER IS SPEEDING UP AS THIS LOOP GOES
		} catch(Exception e) {
			//System.out.println("exception: " + e);
			break;
		}
	}
			}
		}
	}
	/**
	 * Primitive implementation for CLEAN
	 */
	public final LogoObject pCLEAN( InterpEnviron interp, LogoObject[] params)throws LanguageException {
		testNumParams(params, 0);
		synchronized (this) {
			if (getTurtlePanel() != null) {
				getTurtlePanel().reset();
			}
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for CLEARSCREEN
	 */
	public final LogoObject pCLEARSCREEN( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		synchronized( this ) {
			clearScreenIMP();
		}
		return LogoVoid.obj;
	}
	public final void clearScreenIMP() throws LanguageException {
		clearTextIMP();
		clearGraphicsIMP();
		resetTurtleIMP();
	}
	public final void clearTextIMP() throws LanguageException {
		if (getTurtlePanel() != null) {
			getTurtlePanel().setText("");
		}
	}
	public final void clearGraphicsIMP() throws LanguageException {
		if (getTurtlePanel() != null) {
			getTurtlePanel().reset();
		}
	}
	public final void resetTurtleIMP() throws LanguageException {
		_xpos = 0;
		_ypos = 0;
		_heading = 0;
		updateTurtle();
	}
	
	/**
	 * Primitive implementation for CLEARGRAPHICS
	 */
	public final LogoObject pCLEARGRAPHICS( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		synchronized (this) {
			clearGraphicsIMP();
			resetTurtleIMP();
		}
		return LogoVoid.obj;
	}
		/**
	 * Primitive implementation for CLEARTEXT
	 */
	public final LogoObject pCLEARTEXT( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		synchronized (this) {
			clearTextIMP();
			updateTurtle();
		}
		return LogoVoid.obj;
	}
	
	/**
	 * Primitive implementation for DISTANCETO
	 */
	public final LogoObject pDISTANCETO( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		if (!(params[0] instanceof LogoList)) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-424"+" "+"Position list expected");
				}
				else {
					throw new LanguageException("Position list expected");
				}
		}
		if (params[0].length() != 2) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-433"+" "+"Position list expected");
				}
				else {
					throw new LanguageException("Position list expected");
				}
		}
		double xval = ((LogoList)(params[0])).pickInPlace(0).toNumber();
		double yval = ((LogoList)(params[0])).pickInPlace(1).toNumber();
		double dist;
		synchronized (this) {
			dist = Math.sqrt((xval-_xpos)*(xval-_xpos)+(yval-_ypos)*(yval-_ypos));
		}
		return new LogoWord(dist);
	}
	/**
	 * Primitive implementation for DISTANCETOXY
	 */
	public final LogoObject pDISTANCETOXY( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 2);
		double xval = params[0].toNumber();
		double yval = params[1].toNumber();
		double dist;
		synchronized (this) {
			dist = Math.sqrt((xval-_xpos)*(xval-_xpos)+(yval-_ypos)*(yval-_ypos));
		}
		return new LogoWord(dist);
	}
	/**
	 * Primitive implementation for DRAW
	 */
	public final LogoObject pDRAW( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		int xs, ys;
		try {
			System.out.println("params for the pDRAW: " + params+"<:|:>"+"virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-466");
			if( params != null ) {
				System.out.println("params for the pDRAW: length" + params.length+"<:|:>"+"virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-468");
				for( int i = 0; i < params.length;i++ ) {
					System.out.println("param " + i + " is " + params[i]+"<:|:>"+"virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-470");
				}
			}
		}
		catch( Exception e ) {
			System.out.println(" EXCEPTION in pDRAW() " + e +"<:|:>"+"virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-475");
		}
		
		if (params.length == 2) {
			xs = params[0].toInteger();
			ys = params[1].toInteger();
		}
		else if (params.length == 0) {
			xs = _xhalfsize;
			ys = _yhalfsize;
		}
		else {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-489"+" "+"Wrong number of arguments");
				}
				else {
					throw new LanguageException("Wrong number of arguments");
				}
		}
		readyTheTurtle( xs, ys );
		
		return LogoVoid.obj;
	}
	public void readyTheTurtle( final int xs, final int ys ) {
		synchronized( this ) {
			_xhalfsize = xs;
			_yhalfsize = ys;
			if (getTurtlePanel() != null) {
				getTurtlePanel().reset();
			}
			else {
//				try {
//					_turtlePanel = new TurtlePanel( _xhalfsize, _yhalfsize );
//					_turtleFrame = makeTurtleFrame( getTurtlePanel(), _xhalfsize*2, _yhalfsize*2 );
//					getTurtlePanel().init();
//					_turtleDisplayThread = new TurtleDisplayThread();
//					_turtleDisplayThread.setTurtleWindow( getTurtlePanel() );
//					getTurtlePanel().setTurtleDisplayThread( _turtleDisplayThread );
//					_turtleDisplayThread.start();
//				}
//				catch (InternalError e) {}
			}
			
			getTurtlePanel().setBackColor( Color.white );
			getTurtlePanel().setPenColor( Color.black );
			_penState = PS_DOWN;
			_xpos = 0;
			_ypos = 0;
			_heading = 0;
			_turtleVisible = true;
			_wrapState = WS_WRAP;
			updateTurtle();
		}
	}
	/**
	 * Primitive implementation for FENCE
	 */
	public final LogoObject pFENCE( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		synchronized (this) {
			if (_xpos < -_xhalfsize || _xpos > _xhalfsize-1 ||
				_ypos < -_yhalfsize || _ypos > _yhalfsize-1)
			{
				
					if( LanguageException.useVerboseMessages() ) {
						throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-541"+" "+"Turtle out of bounds in FENCE");
					}
					else {
						throw new LanguageException("Turtle out of bounds in FENCE");
					}
			}
			_wrapState = WS_FENCE;
			updateTurtle();
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for FORWARD
	 */
	public final LogoObject pFORWARD( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		double dist = params[0].toNumber();
		moveAmount(dist);
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for GETBACKGROUND
	 */
	public final LogoObject pGETBACKGROUND( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		LogoObject[] a = new LogoObject[3];
		
		final Color backColor = getTurtlePanel().getBackColor();
		
		a[0] = new LogoWord(backColor.getRed());
		a[1] = new LogoWord(backColor.getGreen());
		a[2] = new LogoWord(backColor.getBlue());
		return new LogoList(a);
	}
	/**
	 * Primitive implementation for GETPENCOLOR
	 */
	public final LogoObject pGETPENCOLOR( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		LogoObject[] a = new LogoObject[3];
		final Color penColor = getTurtlePanel().getPenColor();
		a[0] = new LogoWord(penColor.getRed());
		a[1] = new LogoWord(penColor.getGreen());
		a[2] = new LogoWord(penColor.getBlue());
		return new LogoList(a);
	}
	/**
	 * Primitive implementation for HEADING
	 */
	public final LogoObject pHEADING( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		double temp;
		synchronized (this) {
			temp = _heading;
		}
		return new LogoWord(temp);
	}
	/**
	 * Primitive implementation for HIDETURTLE
	 */
	public final LogoObject pHIDETURTLE( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		synchronized (this) {
			_turtleVisible = false;
			updateTurtle();
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for HOME
	 */
	public final LogoObject pHOME( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		synchronized (this) {
			drawLineTo(0, 0);
			_xpos = 0;
			_ypos = 0;
			_heading = 0;
			updateTurtle();
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for LABEL
	 */
	public final LogoObject pLABEL( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		String str = null;
		if (params[0] instanceof LogoList) {
			str = ((LogoList)(params[0])).toStringOpen();
		}
		else {
			str = params[0].toString();
		}
		synchronized (this) {
			if (getTurtlePanel() != null && _penState != PS_UP) {
				if (_penState == PS_DOWN) {
					getTurtlePanel()._graphics.setColor(getTurtlePanel().getPenColor());
				}
				else {
					getTurtlePanel()._graphics.setColor(getTurtlePanel().getBackColor());
				}
				getTurtlePanel()._graphics.drawString(str, (int)(Math.round(_xhalfsize+_xpos)), (int)(Math.round(_yhalfsize-_ypos)));
				getTurtlePanel().redisplay();
			}
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for LEFT
	 */
	public final LogoObject pLEFT( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		double val = params[0].toNumber();
		turnAmount(-val);
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for NODRAW
	 */
	public final LogoObject pNODRAW( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
//		synchronized (this) {
//			if (_turtleFrame != null) {
//				_turtleFrame.dispose();
//				_turtleFrame = null;
//				_turtlePanel = null;
//				getTurtlePanel()._graphics = null;
//				_turtleDisplayThread.kill();
//				_turtleDisplayThread = null;
//				getTurtlePanel()._redisplayInterval = 0;
//			}
//		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for PALETTE
	 */
	public final LogoObject pPALETTE( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		if (!(params[0] instanceof LogoWord)) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-684"+" "+"Color name expected");
				}
				else {
					throw new LanguageException("Color name expected");
				}
		}
		Color color = (Color)(_colorNames.get(params[0].toCaselessString()));
		if (color == null) {
			return new LogoList();
		}
		LogoObject[] a = new LogoObject[3];
		a[0] = new LogoWord(color.getRed());
		a[1] = new LogoWord(color.getGreen());
		a[2] = new LogoWord(color.getBlue());
		return new LogoList(a);
	}
	/**
	 * Primitive implementation for PALETTEP
	 */
	public final LogoObject pPALETTEP( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		if (!(params[0] instanceof LogoWord)) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-708"+" "+"Color name expected");
				}
				else {
					throw new LanguageException("Color name expected");
				}
		}
		return new LogoWord(_colorNames.get(params[0].toCaselessString()) != null);
	}
	/**
	 * Primitive implementation for PENDOWN
	 */
	public final LogoObject pPENDOWN( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		pPENDOWN_IMP();
		return LogoVoid.obj;
	}
	public final void pPENDOWN_IMP() {
		synchronized (this) {
			_penState = PS_DOWN;
		}
	}
	/**
	 * Primitive implementation for PENERASE
	 */
	public final LogoObject pPENERASE( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		synchronized (this) {
			_penState = PS_ERASE;
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for PENUP
	 */
	public final LogoObject pPENUP( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		synchronized (this) {
			_penState = PS_UP;
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for POS
	 */
	public final LogoObject pPOS( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		double val1;
		double val2;
		synchronized (this) {
			val1 = _xpos;
			val2 = _ypos;
		}
		LogoObject[] a = new LogoObject[2];
		a[0] = new LogoWord(val1);
		a[1] = new LogoWord(val2);
		return new LogoList(a);
	}
	/**
	 * Primitive implementation for REFRESH
	 */
	public final LogoObject pREFRESH( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		synchronized(this) {
			if (getTurtlePanel() != null) {
				getTurtlePanel().paint( getTurtlePanel().getGraphics() );
			}
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for REFRESHINTERVAL
	 */
	public final LogoObject pREFRESHINTERVAL( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		synchronized(this) {
			if (getTurtlePanel() != null) {
				getTurtlePanel().setRedisplayInterval( params[0].toInteger() );
				_turtleDisplayThread.setInterval(getTurtlePanel().getRedisplayInterval());
			}
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for RESETPALETTE
	 */
	public final LogoObject pRESETPALETTE( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		resetPalette();
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for RIGHT
	 */
	public final LogoObject pRIGHT( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		double val = params[0].toNumber();
		turnAmount(val);
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for SETBACKGROUND
	 */
	public final LogoObject pSETBACKGROUND( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		Color temp = findColor(params[0]);
		synchronized (this) {
			getTurtlePanel().setBackColor( temp );
		}
		return LogoVoid.obj;
	}
	/**
		Primitive implementation for FILL
		Takes two params, both must be lists of the same length.
		
		p1: the x points
		p2: the y points
	 */
	public final LogoObject pFILL( InterpEnviron interp, LogoObject[] ps) throws LanguageException {
		testNumParams(ps, 2);
		
		if( !(ps[0] instanceof LogoList) && !(ps[1] instanceof LogoList) ) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-831"+" "+"FILL expects both parameters to be lists, the first list represents the X points, the second list represents the Y points.");
				}
				else {
					throw new LanguageException("FILL expects both parameters to be lists, the first list represents the X points, the second list represents the Y points.");
				}
		}
		final LogoList xs = (LogoList)ps[0];
		final LogoList ys = (LogoList)ps[1];
		
		
		
		
		if( xs.length() != ys.length() ) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-846"+" "+"FILL expects the same length for the X points list and the Y points list.");
				}
				else {
					throw new LanguageException("FILL expects the same length for the X points list and the Y points list.");
				}
		}
		
		if (_penState == PS_DOWN) {
			getTurtlePanel()._graphics.setColor(getTurtlePanel().getPenColor());
		} else {
			// since pen is up, don't do any drawing
			return LogoVoid.obj;
		}
		
		synchronized (this) {
			final Graphics g = getTurtlePanel()._graphics;
			final int n = xs.length();
			
			final int[] xps = new int[n];
			final int[] yps = new int[n];
			
			for( int i = 0; i < n; i++ ) {
				
				// damn logo is ONE based not ZERO based!
				final LogoObject xo = xs.pick(i+1);
				final LogoObject yo = ys.pick(i+1);
				
				// SET THE POINTS, ALSO, FLIP AND ADJUST SO COORDINATE SYSTEM 
				// IS IN CENTER AND Y IS POSITIVE GOING UP
				xps[i]= xo.toInteger() + _xhalfsize;
				yps[i]= -(yo.toInteger() - _yhalfsize);
				
				
				
			}
			
			g.fillPolygon( xps, yps, n );
			
			updateTurtle();
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for SETHEADING
	 */
	public final LogoObject pSETHEADING( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		double val = params[0].toNumber();
		synchronized (this) {
			_heading = fixAngle(val);
			updateTurtle();
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for SETPALETTE
	 */
	public final LogoObject pSETPALETTE( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 2);
		if (!(params[0] instanceof LogoWord)) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-908"+" "+"Color name expected");
				}
				else {
					throw new LanguageException("Color name expected");
				}
		}
		_colorNames.put(params[0].toCaselessString(), findColor(params[1]));
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for SETPENCOLOR
	 */
	public final LogoObject pSETPENCOLOR( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		Color temp = findColor(params[0]);
		synchronized (this) {
			getTurtlePanel().setPenColor( temp );
		}
		return LogoVoid.obj;
	}
	public final LogoObject pSETFONTCOLOR( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		Color temp = findColor(params[0]);
		synchronized (this) {
			getTurtlePanel().setForeground( temp );
		}
		return LogoVoid.obj;
	}
	public final LogoObject pGETFONTCOLOR( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		LogoObject[] a = new LogoObject[3];
		final Color fontColor = getTurtlePanel().getForeground();
		a[0] = new LogoWord(fontColor.getRed());
		a[1] = new LogoWord(fontColor.getGreen());
		a[2] = new LogoWord(fontColor.getBlue());
		return new LogoList(a);
	}
	public final LogoObject pGETFONTSIZE( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		int fontSize = 0;
		synchronized( this ) {
			fontSize = getTurtlePanel().getFont().getSize();
		}
		return new LogoWord(fontSize);
	}
	public final LogoObject pSETFONTSIZE( InterpEnviron interp, LogoObject[] params)  throws LanguageException {
		testNumParams(params, 1);
		double val = params[0].toNumber();
		synchronized (this) {
			final int size = (int)val;
			final Font font = getTurtlePanel().getFont();
			final String name = font.getName();
			final int style = font.getStyle();
			final Font newFont = new Font( name, style, size );
			getTurtlePanel().setFont( newFont );
		}
		return LogoVoid.obj;
	}
	private int _shapeIndex = 0;
	public final LogoObject pSETSHAPE( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params,1);
		synchronized (this) {
			_shapeIndex = (int)params[0].toNumber();
		}
		return LogoVoid.obj;
	}
	public final LogoObject pSTAMP( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params,0);
		synchronized (this) {
			stamp(_shapeIndex);
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for SETPOS
	 */
	public final LogoObject pSETPOS( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		if (!(params[0] instanceof LogoList)) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-989"+" "+"Position list expected");
				}
				else {
					throw new LanguageException("Position list expected");
				}
		}
		if (params[0].length() != 2) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-998"+" "+"Position list expected");
				}
				else {
					throw new LanguageException("Position list expected");
				}
		}
		double xval = ((LogoList)(params[0])).pickInPlace(0).toNumber();
		double yval = ((LogoList)(params[0])).pickInPlace(1).toNumber();
		synchronized (this) {
			moveTo(xval, yval);
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for SETX
	 */
	public final LogoObject pSETX( InterpEnviron interp, LogoObject[] params)  throws LanguageException {
		testNumParams(params, 1);
		double val = params[0].toNumber();
		synchronized (this) {
			if (_wrapState == WS_FENCE && (val < -_xhalfsize || val > _xhalfsize-1)) {
				
					if( LanguageException.useVerboseMessages() ) {
						throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1021"+" "+"Turtle out of bounds");
					}
					else {
						throw new LanguageException("Turtle out of bounds");
					}
			}
			if (_wrapState == WS_WRAP) {
				while (true) {
					if (val < -_xhalfsize-0.5) {
						val = wrapXMinus(val, _ypos);
					}
					else if (val > _xhalfsize-0.5) {
						val = wrapXPlus(val, _ypos);
					}
					else {
						break;
					}
				}
			}
			drawLineTo(val, _ypos);
			_xpos = val;
			updateTurtle();
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for SETXY
	 */
	public final LogoObject pSETXY( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 2);
		double xval = params[0].toNumber();
		double yval = params[1].toNumber();
		synchronized (this) {
			moveTo(xval, yval);
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for SETY
	 */
	public final LogoObject pSETY( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		double val = params[0].toNumber();
		synchronized (this) {
			if (_wrapState == WS_FENCE && (val < -_yhalfsize || val > _yhalfsize-1)) {
				
					if( LanguageException.useVerboseMessages() ) {
						throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1068"+" "+"Turtle out of bounds");
					}
					else {
						throw new LanguageException("Turtle out of bounds");
					}
			}
			if (_wrapState == WS_WRAP) {
				while (true) {
					if (val < -_yhalfsize-0.5) {
						val = wrapYMinus(_xpos, val);
					}
					else if (val > _yhalfsize-0.5) {
						val = wrapYPlus(_xpos, val);
					}
					else {
						break;
					}
				}
			}
			drawLineTo(_xpos, val);
			_ypos = val;
			updateTurtle();
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for SHOWTURTLE
	 */
	public final LogoObject pSHOWTURTLE( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		synchronized (this) {
			_turtleVisible = true;
			updateTurtle();
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for TOWARDS
	 */
	public final LogoObject pTOWARDS( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		if (!(params[0] instanceof LogoList)) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1112"+" "+"Position list expected");
				}
				else {
					throw new LanguageException("Position list expected");
				}
		}
		if (params[0].length() != 2) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1121"+" "+"Position list expected");
				}
				else {
					throw new LanguageException("Position list expected");
				}
		}
		double xval = ((LogoList)(params[0])).pickInPlace(0).toNumber();
		double yval = ((LogoList)(params[0])).pickInPlace(1).toNumber();
		double ang;
		synchronized (this) {
			ang = fixAngle(90.0 - Math.atan2(yval-_ypos, xval-_xpos)/Math.PI*180);
		}
		return new LogoWord(ang);
	}
	/**
	 * Primitive implementation for TOWARDSXY
	 */
	public final LogoObject pTOWARDSXY( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 2);
		double xval = params[0].toNumber();
		double yval = params[1].toNumber();
		double ang;
		synchronized (this) {
			ang = fixAngle(90.0 - Math.atan2(yval-_ypos, xval-_xpos)/Math.PI*180);
		}
		return new LogoWord(ang);
	}
	/**
	 * Primitive implementation for UNSETPALETTE
	 */
	public final LogoObject pUNSETPALETTE( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 1);
		if (!(params[0] instanceof LogoWord)) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1156"+" "+"Color name expected");
				}
				else {
					throw new LanguageException("Color name expected");
				}
		}
		_colorNames.remove(params[0].toCaselessString());
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for WINDOW
	 */
	public final LogoObject pWINDOW( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		synchronized (this) {
			_wrapState = WS_WINDOW;
			updateTurtle();
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for WRAP
	 */
	public final LogoObject pWRAP( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		synchronized (this) {
			_xpos = wrapCoordinate(_xpos, _xhalfsize);
			_ypos = wrapCoordinate(_ypos, _yhalfsize);
			_wrapState = WS_WRAP;
			updateTurtle();
		}
		return LogoVoid.obj;
	}
	/**
	 * Primitive implementation for XCOR
	 */
	public final LogoObject pXCOR( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		double val;
		synchronized (this) {
			val = _xpos;
		}
		return new LogoWord(val);
	}
	/**
	 * Primitive implementation for XSIZE
	 */
	public final LogoObject pXSIZE( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		double val;
		synchronized (this) {
			val = _xhalfsize;
		}
		return new LogoWord(val);
	}
	/**
	 * Primitive implementation for YCOR
	 */
	public final LogoObject pYCOR( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		double val;
		synchronized (this) {
			val = _ypos;
		}
		return new LogoWord(val);
	}
	/**
	 * Primitive implementation for YSIZE
	 */
	public final LogoObject pYSIZE( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		double val;
		synchronized (this) {
			val = _yhalfsize;
		}
		return new LogoWord(val);
	}
	/**
		Tue Jul 26 00:14:22 CDT 2005
			Added primitive implementation for TILES.
			
			What is it?
				it brings up a window that the user can resize to see the turtle graphics image as it would appear as a tiled image (say a desktop background)
	 */
	public final LogoObject pTILES( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		synchronized (this) {
			TurtleTilesPanel.showInFrameWithThread( getTurtlePanel() );
		}
		return LogoVoid.obj;
	}
	/**
		Get color from a LogoObject
		@param param the object to convert
		@return the color
		@exception virtuoso.logo.LanguageException couldn't convert
	*/
	private final Color findColor(LogoObject param)throws LanguageException {
		//System.out.println("findColor( LogoObject param: " + param + ")"+"<:|:>"+"virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1254");
		if( param instanceof LogoWord ) {
			//System.out.println("param is logoword"+"<:|:>"+"virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1256");
			Color colorVal = (Color)(_colorNames.get(param.toCaselessString()));
			if (colorVal == null) {
				
					if( LanguageException.useVerboseMessages() ) {
						throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1261"+" "+"Color not found");
					}
					else {
						throw new LanguageException("Color not found");
					}
			}
			return colorVal;
		}
		else {
			
			//System.out.println("param is not logo word, its length is: " + param.length()+"<:|:>"+"virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1271");
			//System.out.println("the types are"+"<:|:>"+"virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1272");
			final int count = param.length();
			for( int i = 0; i < count; i++ ) {
				try {
					//System.out.println(i+" is : " + ((LogoList)param).pickInPlace( i ).getClass()+"<:|:>"+"virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1276");
				}
				catch( Exception e ) {
					System.out.println( i+" could not do type because of exception " + e +"<:|:>"+"virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1279");
				}
			}
			
			
			if( param.length() != 3 ) {
				
					if( LanguageException.useVerboseMessages() ) {
						throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1287"+" "+"Three elements expected in color list");
					}
					else {
						throw new LanguageException("Three elements expected in color list");
					}
			}
			int r=0;
			int g=0;
			int b=0;
			try {
				//System.out.println("red pick is: " + ((LogoList)param).pickInPlace(0)+"<:|:>"+"virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1297");
				r = ((LogoList)param).pickInPlace(0).toInteger();
			}
			catch( LanguageException e ) {
				
					if( LanguageException.useVerboseMessages() ) {
						throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1303"+" "+e.getMessage() + " in red value of color list");
					}
					else {
						throw new LanguageException(e.getMessage() + " in red value of color list");
					}
			}
			try {
				//System.out.println("green pick is: " + ((LogoList)param).pickInPlace(1)+"<:|:>"+"virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1310");
				g = ((LogoList)param).pickInPlace(1).toInteger();
			}
			catch (LanguageException e) {
				
					if( LanguageException.useVerboseMessages() ) {
						throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1316"+" "+e.getMessage() + " in green value of color list");
					}
					else {
						throw new LanguageException(e.getMessage() + " in green value of color list");
					}
			}
			try {
				//System.out.println("blue pick is: " + ((LogoList)param).pickInPlace(2)+"<:|:>"+"virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1323");
				b = ((LogoList)param).pickInPlace(2).toInteger();
			}
			catch (LanguageException e) {
				
					if( LanguageException.useVerboseMessages() ) {
						throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1329"+" "+e.getMessage() + " in blue value of color list");
					}
					else {
						throw new LanguageException(e.getMessage() + " in blue value of color list");
					}
			}
			if (r < 0 || r > 255) {
				
					if( LanguageException.useVerboseMessages() ) {
						throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1338"+" "+"Bad color (red value out of range)");
					}
					else {
						throw new LanguageException("Bad color (red value out of range)");
					}
			}
			if (g < 0 || g > 255) {
				
					if( LanguageException.useVerboseMessages() ) {
						throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1347"+" "+"Bad color (green value out of range)");
					}
					else {
						throw new LanguageException("Bad color (green value out of range)");
					}
			}
			if (b < 0 || b > 255) {
				
					if( LanguageException.useVerboseMessages() ) {
						throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1356"+" "+"Bad color (blue value out of range)");
					}
					else {
						throw new LanguageException("Bad color (blue value out of range)");
					}
			}
			return new Color(r, g, b);
		}
	}
	/**
	 * Update graphics window's turtle info
	 */
	public final void updateTurtle() {
		if (getTurtlePanel() != null) {
			getTurtlePanel().setTurtleState(_xpos, _ypos, (90.0-_heading)/180.0*Math.PI, _turtleVisible, _wrapState == WS_WRAP);
		}
	}
	/**
	 * Draw a line in the graphics context
	 */
	private final void drawLineTo( double x2, double y2) {
		if (getTurtlePanel() != null && _penState != PS_UP) {
			if (_penState == PS_DOWN) {
				getTurtlePanel()._graphics.setColor(getTurtlePanel().getPenColor());
			}
			else {
				getTurtlePanel()._graphics.setColor(getTurtlePanel().getBackColor());
			}
			getTurtlePanel()._graphics.drawLine((int)(Math.round(_xhalfsize+_xpos)),
				(int)(Math.round(_yhalfsize-_ypos)),
				(int)(Math.round(_xhalfsize+x2)),
				(int)(Math.round(_yhalfsize-y2)));
		}
	}
	
	//private Polygon[] _polygons = new Polygon[3]; {
	//	_polygon
	//}
	private final void stamp(final int index) {
		if( index == 0 ) {
			return;
		} 
		
		if(getTurtlePanel() != null && _penState != PS_UP) {
		
			final Graphics g = getTurtlePanel()._graphics;
			if (_penState == PS_DOWN) {
				g.setColor(getTurtlePanel().getPenColor());
			} else {
				g.setColor(getTurtlePanel().getBackColor());
			}
			
			
			final int x = (int)(Math.round(_xhalfsize+_xpos));
			final int y = (int)(Math.round(_yhalfsize-_ypos));
			
			if( index == 1 ) {
				final int shift = 1;
				final int size = 3;
				g.fillRect(x-shift,y-shift, size, size);
			} else if( index == 2 ) {
				final int shift = 2;
				final int size = 5;
				g.fillRect(x-shift,y-shift, size, size);
			} else if( index == 3 ) {
				// draw the cross thinggy
				final int size = 20;
				final int halfSize = size/2;
				
				// DRAW VERTICAL
				g.drawLine( x - 1, y - halfSize, x -1, y + halfSize );
				g.drawLine( x , y - halfSize-1, x,  y + halfSize +1 );
				g.drawLine( x + 1, y - halfSize, x +1, y + halfSize );
				// DRAW HORIZONTAL
				g.drawLine( x - halfSize, y - 1, x + halfSize, y - 1 );
				g.drawLine( x - halfSize-1, y , x + halfSize+1, y );
				g.drawLine( x - halfSize, y + 1, x + halfSize, y + 1 );
				
				
			}
			
		}
		
	}
	
	/**
	 * Wrap coordinate into window bounds
	 *
	 * @param val value
	 * @param limit value limit
	 *
	 * @return wrapped value
	 */
	private final double wrapCoordinate( double val, double limit) {
		double limit2 = limit*2.0;
		double ret;
		
		if (val < 0) {
			ret = val + (limit2*Math.floor((-val)/limit2)) + limit2;
		}
		else {
			ret = val - (limit2*Math.floor(val/limit2));
		}
		if (ret > limit-0.5) {
			return ret-limit2;
		}
		else {
			return ret;
		}
	}
	/**
	 * Fix angle (make it between 0 and 360)
	 *
	 * @param ang angle
	 *
	 * @return fixed angle
	 */
	private final double fixAngle( double ang) {
		if (ang < 0) {
			return ang + (360.0*Math.floor((-ang)/360.0)) + 360.0;
		}
		else {
			return ang - (360.0*Math.floor(ang/360.0));
		}
	}
	/**
	 * Setxy wrap helper
	 *
	 * @param xval x coordinate
	 * @param yval y coordinate
	 *
	 * @return new xval
	 */
	private final double wrapXMinus( double xval, double yval) {
		double newy = _ypos+(-_xhalfsize-0.5-_xpos)/(xval-_xpos)*(yval-_ypos);
		drawLineTo(-_xhalfsize-0.5, newy);
		_xpos = _xhalfsize-0.5;
		_ypos = newy;
		updateTurtle();
		return xval + _xhalfsize*2.0;
	}
	/**
	 * Setxy wrap helper
	 *
	 * @param xval x coordinate
	 * @param yval y coordinate
	 *
	 * @return new yval
	 */
	private final double wrapYMinus( double xval, double yval) {
		double newx = _xpos+(-_yhalfsize-0.5-_ypos)/(yval-_ypos)*(xval-_xpos);
		drawLineTo(newx, -_yhalfsize-0.5);
		_ypos = _yhalfsize-0.5;
		_xpos = newx;
		updateTurtle();
		return yval + _yhalfsize*2.0;
	}
	/**
	 * Setxy wrap helper
	 *
	 * @param xval x coordinate
	 * @param yval y coordinate
	 *
	 * @return new xval
	 */
	private final double wrapXPlus( double xval, double yval) {
		double newy = _ypos+(_xhalfsize-0.5-_xpos)/(xval-_xpos)*(yval-_ypos);
		drawLineTo(_xhalfsize-0.5, newy);
		_xpos = -_xhalfsize-0.5;
		_ypos = newy;
		updateTurtle();
		return xval - _xhalfsize*2.0;
	}
	/**
	 * Setxy wrap helper
	 *
	 * @param xval x coordinate
	 * @param yval y coordinate
	 *
	 * @return new yval
	 */
	private final double wrapYPlus( double xval, double yval) {
		double newx = _xpos+(_yhalfsize-0.5-_ypos)/(yval-_ypos)*(xval-_xpos);
		drawLineTo(newx, _yhalfsize-0.5);
		_ypos = -_yhalfsize-0.5;
		_xpos = newx;
		updateTurtle();
		return yval - _yhalfsize*2.0;
	}
	/**
	 * Move to x and y coordinate
	 *
	 * @param xval x coordinate
	 * @param yval y coordinate
	 *
	 * @exception virtuoso.logo.TurtleBoundsException turtle out of bounds
	 */
	private static final boolean _TURTLE_LIMIT_FLAG = true;
	private static final int _X_TURTLE_RECT_LIMIT = 99256;
	private static final int _Y_TURTLE_RECT_LIMIT = 99256;
	private final void moveTo( double xval, double yval) throws LanguageException {
		if( _wrapState == WS_FENCE
		    && (xval < -_xhalfsize || xval > _xhalfsize-1
		       || yval < -_yhalfsize || yval > _yhalfsize-1
		       )
		  )
		{
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1565"+" "+"Turtle out of bounds");
				}
				else {
					throw new LanguageException("Turtle out of bounds");
				}
		}
		// ==================================
		// ADDED Thu Sep 08 00:19:51 CDT 2005
		// make a limiter on the range the
		// turtle can be located, only if the
		// flag is turned on...
		if( _TURTLE_LIMIT_FLAG ) {
			// DO THE CHECK AS A RECTANGLE
			if( xval < -_X_TURTLE_RECT_LIMIT || xval > _X_TURTLE_RECT_LIMIT
			 || yval < -_Y_TURTLE_RECT_LIMIT || yval > _Y_TURTLE_RECT_LIMIT
			) {
				
					if( LanguageException.useVerboseMessages() ) {
						throw new LanguageException("virtuoso.logo.lib.TurtlePrimitives.java.ux:TL-1583"+" "+"Turtle out of bounds");
					}
					else {
						throw new LanguageException("Turtle out of bounds");
					}
			}
		}
		// ==================================
		if (_wrapState == WS_WRAP) {
			while (true) {
				if (xval < -_xhalfsize-0.5) {
					if (yval < -_yhalfsize-0.5) {
						if ((-_xhalfsize-0.5-_xpos)*(yval-_ypos) > (-_yhalfsize-0.5-_ypos)*(xval-_xpos)) {
							yval = wrapYMinus(xval, yval);
						}
						else {
							xval = wrapXMinus(xval, yval);
						}
					}
					else if (yval > _yhalfsize-0.5) {
						if ((-_xhalfsize-0.5-_xpos)*(yval-_ypos) < (_yhalfsize-0.5-_ypos)*(xval-_xpos)) {
							yval = wrapYPlus(xval, yval);
						}
						else {
							xval = wrapXMinus(xval, yval);
						}
					}
					else {
						xval = wrapXMinus(xval, yval);
					}
				}
				else if (xval > _xhalfsize-0.5) {
					if (yval < -_yhalfsize-0.5) {
						if ((_xhalfsize-0.5-_xpos)*(yval-_ypos) < (-_yhalfsize-0.5-_ypos)*(xval-_xpos)) {
							yval = wrapYMinus(xval, yval);
						}
						else {
							xval = wrapXPlus(xval, yval);
						}
					}
					else if (yval > _yhalfsize-0.5) {
						if ((_xhalfsize-0.5-_xpos)*(yval-_ypos) > (_yhalfsize-0.5-_ypos)*(xval-_xpos)) {
							yval = wrapYPlus(xval, yval);
						}
						else {
							xval = wrapXPlus(xval, yval);
						}
					}
					else {
						xval = wrapXPlus(xval, yval);
					}
				}
				else {
					if (yval < -_yhalfsize-0.5) {
						yval = wrapYMinus(xval, yval);
					}
					else if (yval > _yhalfsize-0.5) {
						yval = wrapYPlus(xval, yval);
					}
					else {
						break;
					}
				}
			}
		}
		drawLineTo(xval, yval);
		_xpos = xval;
		_ypos = yval;
		updateTurtle();
	}
	/**
	 * Reset color palette
	 */
	private final void resetPalette() {
		_colorNames = new Hashtable();
		
		_colorNames.put(new CaselessString("0"), Color.black);
		_colorNames.put(new CaselessString("1"), Color.blue);
		_colorNames.put(new CaselessString("2"), Color.green);
		_colorNames.put(new CaselessString("3"), Color.cyan);
		_colorNames.put(new CaselessString("4"), Color.red);
		_colorNames.put(new CaselessString("5"), Color.magenta);
		_colorNames.put(new CaselessString("6"), Color.yellow);
		_colorNames.put(new CaselessString("7"), Color.white);
		
		_colorNames.put(new CaselessString("black"), Color.black);
		_colorNames.put(new CaselessString("blue"), Color.blue);
		_colorNames.put(new CaselessString("cyan"), Color.cyan);
		_colorNames.put(new CaselessString("darkgray"), Color.darkGray);
		_colorNames.put(new CaselessString("gray"), Color.gray);
		_colorNames.put(new CaselessString("green"), Color.green);
		_colorNames.put(new CaselessString("lightgray"), Color.lightGray);
		_colorNames.put(new CaselessString("magenta"), Color.magenta);
		_colorNames.put(new CaselessString("orange"), Color.orange);
		_colorNames.put(new CaselessString("pink"), Color.pink);
		_colorNames.put(new CaselessString("red"), Color.red);
		_colorNames.put(new CaselessString("white"), Color.white);
		_colorNames.put(new CaselessString("yellow"), Color.yellow);
	}

}