
/* ***************************************************
	FILE:  ZZPrimitives.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.logo.lib;

// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// turtle_tracks_imports
import virtuoso.logo.*;
import virtuoso.logo.app.*;
import virtuoso.logo.app.actions.*;
import virtuoso.logo.lib.*;
import virtuoso.logo.file.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.text.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
import java.awt.image.*;
import javax.imageio.ImageIO;

/**
 * Names of primitives (keywords)
 */
public final class ZZPrimitives extends PrimitiveGroup
{
	private static final double _radToDeg = 180.0 / Math.PI;
	private static final double _degToRad = Math.PI / 180.0;

	private GuiConsole _guiConsole;
	private GuiConsole getGuiConsole() { return _guiConsole; }

	/**
	 * Set up primitive group
	 */
	protected void setup(Machine mach,virtuoso.logo.Console console)throws SetupException {
		if( console instanceof GuiConsole ) {
			_guiConsole = (GuiConsole)console;
			//System.out.println("guiconsole stored in ZZPrimitives"+"<:|:>"+"virtuoso.logo.lib.ZZPrimitives.java.ux:TL-85");
		}
		
		registerPrimitive("ISFRACTION", "pISFRACTION",1);
		registerPrimitive("ISFRAC", "pISFRACTION",1);
		
		registerPrimitive("FRACTIONNUMERATOR", "pGET_FRACTION_NUMERATOR",1);
		registerPrimitive("NUMERATOR", "pGET_FRACTION_NUMERATOR",1);
		registerPrimitive("FRACNU", "pGET_FRACTION_NUMERATOR",1);
		registerPrimitive("FRACN", "pGET_FRACTION_NUMERATOR",1);
		
		registerPrimitive("FRACTIONDENOMINATOR", "pGET_FRACTION_DENOMINATOR",1);
		registerPrimitive("DENOMINATOR", "pGET_FRACTION_DENOMINATOR",1);
		registerPrimitive("FRACDE", "pGET_FRACTION_DENOMINATOR",1);
		registerPrimitive("FRACD", "pGET_FRACTION_DENOMINATOR",1);
		
		
		registerPrimitive("MEMBER?", "pMEMBERP", 2);
		registerPrimitive("MEMBERP", "pMEMBERP", 2);
		
		registerPrimitive("RC", "pREADCHARACTER", 0);
		registerPrimitive("READCHARACTER", "pREADCHARACTER", 0);
		registerPrimitive("READLIST", "pREADLIST", 0);
		registerPrimitive("RL", "pREADLIST", 0);
		registerPrimitive("RW", "pREADWORD", 0);
		registerPrimitive("READWORD", "pREADWORD", 0);
		
		registerPrimitive("HELP", "pHELP", 1);
		
		
		registerPrimitive("xmresource", "pXMRESOURCE", 1);
		
		//console.putStatusMessage("zz");
		
		boolean jimihere = false;
		try {
			
			jimihere = true;
		} catch( Exception e ) {
			
		}
		if( jimihere ) {
			registerPrimitive( "SAVETOIMAGE", "pSAVETOIMAGE", 1);
		}
		
		
		
	}
	/**
	 * 
	 */
	public final LogoObject pISFRACTION( InterpEnviron interp, LogoObject[] params ) throws LanguageException {
		testNumParams(params, 1);
		if( !(params[0] instanceof LogoWord) ) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.ZZPrimitives.java.ux:TL-141"+" "+"Word expected.");
				}
				else {
					throw new LanguageException("Word expected.");
				}
		}
		final LogoWord word = (LogoWord)params[0];
		//if( !word.isNumber() ) {
		//	
		//}
		final String unparsed = (""+word.unparse()).trim();
		
		
		// FOR THE NUMBER TO BE CONSIDERED A FRACTION IT HAS TO MATCH THIS
		// PATTERN:  ONLY DIGITS AND THEN ONE / AND THEN ONLY DIGITS.
		// also, we allow the first char to be a negative symbol -
		int slashCount = 0;
		final int length = unparsed.length();
		for( int i = 0; i < length; i++ ) {
			final char c = unparsed.charAt(i);
			if( i == 0 && c == '-' ) {
				continue;
			} else if( slashCount == 1 && c == '/' ) {
				// fractions cannot have multiple slashes
				
				return new LogoWord(false);
			} else {
				if( c == '/' ) {
					
					slashCount = 1;
					continue;
				} else {
					if( c == '0' ||c == '1' ||c == '2' ||c == '3' ||c == '4' ||c == '5' ||c == '6' ||c == '7' ||c == '8' ||c == '9' ) {
						continue;
					} else {
						// WE HIT A 'WORD' CHAR THAT IS NOT A NUMBER OR A SLASH
						
						return new LogoWord(false);
					}
				}
			}
		}
		// only a fraction if the slash was found
		
		return new LogoWord(slashCount==1);
	}
	/**
	 * 
	 */
	public final LogoObject pGET_FRACTION_NUMERATOR( InterpEnviron interp, LogoObject[] params ) throws LanguageException {
		final LogoWord isFraction = (LogoWord)pISFRACTION(interp,params);
		if( !isFraction.toBoolean() ) {
			// WE KNOW INDEX ZERO IS OK CAUSE IF NOT ISFRAC WOULD HAVE THREW EXCEPTION FOR US INSTEAD
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.ZZPrimitives.java.ux:TL-196"+" "+"Expected a fraction, not this: " + params[0] );
				}
				else {
					throw new LanguageException("Expected a fraction, not this: " + params[0] );
				}
		}
		final LogoWord word = (LogoWord)params[0];
		final String unparsed = word.unparse();
		final StringBuffer buffer = new StringBuffer();
		final int length = unparsed.length();
		for( int i = 0; i < length; i++ ) {
			final char c = unparsed.charAt(i);
			if( c == '/' ) {
				break;
			} else {
				buffer.append( c );
			}
		}
		return new LogoWord(buffer.toString());
	}
	/**
	 * 
	 */
	public final LogoObject pGET_FRACTION_DENOMINATOR( InterpEnviron interp, LogoObject[] params ) throws LanguageException {
		final LogoWord isFraction = (LogoWord)pISFRACTION(interp,params);
		if( !isFraction.toBoolean() ) {
			// WE KNOW INDEX ZERO IS OK CAUSE IF NOT ISFRAC WOULD HAVE THREW EXCEPTION FOR US INSTEAD
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.ZZPrimitives.java.ux:TL-225"+" "+"Expected a fraction, not this: " + params[0] );
				}
				else {
					throw new LanguageException("Expected a fraction, not this: " + params[0] );
				}
		}
		final LogoWord word = (LogoWord)params[0];
		final String unparsed = word.unparse();
		final StringBuffer buffer = new StringBuffer();
		boolean startAppending=false;
		final int length = unparsed.length();
		for( int i = 0; i < length; i++ ) {
			final char c = unparsed.charAt(i);
			if( startAppending ) {
				buffer.append(c);
			} else if( c == '/' ) {
				startAppending=true;
			}
		}
		return new LogoWord(buffer.toString());
	}
	
		/**
	 * Primitive implementation for MEMBERP
	 */
	public final LogoObject pMEMBERP( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 2);
		final LogoObject a = params[0];
		final LogoObject b = params[1];
		if( b instanceof LogoWord ) {
			final String A = ""+a;
			final String B = ""+b;
			return new LogoWord( B.indexOf( A ) != -1 );
		}
		if (!(params[1] instanceof LogoList)) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.ZZPrimitives.java.ux:TL-262"+" "+"List expected");
				}
				else {
					throw new LanguageException("List expected");
				}
		}
		return new LogoWord(((LogoList)(params[1])).isMember(params[0]));
	}
	/**
	 * Primitive implementation for READCHARACTER
	 */
	public final LogoObject pREADCHARACTER(InterpEnviron interp,LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		//final boolean b = interp.thread().inStream().charAvail();
		//final char c = interp.thread().inStream().getChar();
		//System.out.println("charAvail:[" + b + "] char:["+c+"]"+"<:|:>"+"virtuoso.logo.lib.ZZPrimitives.java.ux:TL-277");
		
		//String line = interp.thread().inStream().getLine();
		//while( line == null || line.length() == 0 ) {
		//	line = interp.thread().inStream().getLine();
		//}
		//return new LogoWord( new CaselessString(line.charAt(0) ) );
		
		GuiConsoleKeyListener.READ_CHAR_MODE = true;
		return new LogoWord(interp.thread().inStream().getLine());
	}
	/**
	 * Primitive implementation for READWORD
	 */
	public final LogoObject pREADWORD( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		return new LogoWord(interp.thread().inStream().getLine());
	}
	/**
	 * Primitive implementation for READLIST
	 */
	public final LogoObject pREADLIST( InterpEnviron interp, LogoObject[] params) throws LanguageException {
		testNumParams(params, 0);
		virtuoso.logo.Console console = null;
		if (interp.thread().inStream() instanceof virtuoso.logo.Console) {
			console = (virtuoso.logo.Console)(interp.thread().inStream());
		}
		StringBuffer cumulative = new StringBuffer();
		char promptChar = '\0';
		Tokenizer tokenizer = new Tokenizer(0);
		while (true) {
			try {
				if (console != null && promptChar != '\0') {
					cumulative.append(console.promptGetLine(promptChar));
				}
				else {
					cumulative.append(interp.thread().inStream().getLine());
				}
				if (cumulative.length() == 0) {
					return new LogoList();
				}
				if (cumulative.charAt(cumulative.length()-1) == '~') {
					cumulative.setCharAt(cumulative.length()-1, ' ');
					
						if( LanguageException.useVerboseMessages() ) {
							throw new LanguageException("virtuoso.logo.lib.ZZPrimitives.java.ux:TL-322"+" "+"Missing ']'");
						}
						else {
							throw new LanguageException("Missing ']'");
						}
					//promptChar = '~';
				}
				else {
					return tokenizer.tokenize(cumulative.toString());
				}
			}
			catch (LanguageException e) {
				promptChar = e.getContChar();
				if (promptChar == '|' || promptChar == '\\') {
					cumulative.append(Machine.LINE_SEPARATOR);
				}
				//else if (promptChar == '~')
				//{
				//	cumulative.append(' ');
				//}
				else {
					throw e;
				}
			}
		}
	}
	/**
		They can type in help command to get help on the command printed to the graphics area.
	 */
	private final HelpDictionary _dictionary = new HelpDictionary();
	public final LogoObject pHELP( InterpEnviron interp, LogoObject[] params ) throws LanguageException {
		testNumParams(params, 1);
		if( !(params[0] instanceof LogoWord) ) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.ZZPrimitives.java.ux:TL-357"+" "+"Word expected.");
				}
				else {
					throw new LanguageException("Word expected.");
				}
		}
		final LogoWord word = (LogoWord)params[0];
		
		
		
		final String unparsed = (""+word.unparse()).trim();
		final String help = _dictionary.getHelp(unparsed);
		
		
		// NEED TO PRINT TO GRAPHICS AREA
		appendTurtleAreaText(help);
		
		return LogoVoid.obj;
	}
	/**
		Save the turtle graphics area out to a jpeg file.
	 */
	public final LogoObject pSAVETOIMAGE( InterpEnviron interp, LogoObject[] params ) throws LanguageException {
		testNumParams(params, 1);
		if( !(params[0] instanceof LogoWord) ) {
			
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.ZZPrimitives.java.ux:TL-384"+" "+"Word expected.");
				}
				else {
					throw new LanguageException("Word expected.");
				}
		}
		final LogoWord word = (LogoWord)params[0];
		
		
		
		String text = (""+word.unparse()).trim();
		
		if( text.toUpperCase().lastIndexOf( ".JPG" ) != text.length()-4 ) {
			text += ".jpg";
		}
		String fileName = text;
		
		
		actuallySaveImage(fileName);
		
		return LogoVoid.obj;
	}
	public final void actuallySaveImage(String fileName){ 
	
		// ALL FILES MUST END WITH .jpg
		if( !fileName.toUpperCase().endsWith( ".JPG" ) ) {
			fileName = fileName + ".jpg";
		}
		
		final File file = new File(fileName);
		if( file.exists() ) {
			// MAYBE SHOULDA WARNED THE USER.
		}
		
		try {
			final Image image = getGuiConsole().getTurtlePanel().getOffscreenImageWithText();
			RenderedImage renderedImage = LogoAid.ensureRenderedImage(image);
			ImageIO.write(renderedImage , "jpg", file);
		} catch (Exception ex) {
			System.out.println("failed to save image: " + ex+"<:|:>"+"virtuoso.logo.lib.ZZPrimitives.java.ux:TL-423");
		}
	}
	
	public final LogoObject pXMRESOURCE( InterpEnviron interp, LogoObject[] params ) throws LanguageException {
		testNumParams(params, 1);
		if( !(params[0] instanceof LogoWord) ) {
				if( LanguageException.useVerboseMessages() ) {
					throw new LanguageException("virtuoso.logo.lib.ZZPrimitives.java.ux:TL-471"+" "+"Word expected.");
				}
				else {
					throw new LanguageException("Word expected.");
				}
		}
		final LogoWord word = (LogoWord)params[0];
		final String zipEntryName = "XM/"+(""+word.unparse()).trim();
		
		// NEED TO PRINT TO GRAPHICS AREA
		appendTurtleAreaText( zipEntryName );
		
		final File zipFile = new File( System.getProperty( "user.dir" ) + File.separator + "XM" );
		System.out.println( "zipFile:" + zipFile );
		System.out.println( " entry name:[" + zipEntryName +"]" );
		final String contents = LogoAid.loadTextFromJarFile( zipFile, zipEntryName  );
		
		LogoAid.showTextInFrame(contents);
		
		return LogoVoid.obj;
	}
	public final void appendTurtleAreaText( final String more ) {
		String text = getGuiConsole().getTurtlePanel().getText();
		if( text != null ) {
			text = text.trim();
		}
		if( text.length() > 0 ) {
			text += "\n";
		}
		getGuiConsole().getTurtlePanel().setText( text + more );
	}
	
}