
/* ***************************************************
	FILE:  ImageLoader.java.ux
	PROJECT:
		Turtle Tracks
	PROGRAMMERS:
		Daniel Azuma (DA)  <dazuma@kagi.com>
		
	Mon Jul 11 15:18:58 CDT 2005
		AUGMENTATION BY  Levi Yourchuck
		Major refactoring to ensue.
		
	COPYRIGHT:
		Copyright (C) 1997-1999  Daniel Azuma  (dazuma@kagi.com)
		
		Copyright (C) 2005 - 2013 Levi Yourchuck (levi_yourchuck@yahoo.com)
		Copyright (C) 2013 -       Levi Yourchuck (levi_yourchuck@yahoo.com)

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.
		
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public
		License along with this program. If not, you can obtain a copy
		by writing to:
			Free Software Foundation, Inc.
			59 Temple Place - Suite 330,
			Boston, MA  02111-1307, USA.
		
	VERSION:
		Turtle Tracks 1.0  (13 November 1999)
	CHANGE HISTORY:
		13 November 1999 -- DA -- Released under GNU General Public License
*************************************************** */
package virtuoso.splash;

import java.io.*;

import java.awt.*;
import java.awt.image.*;
import java.util.*;
import java.util.zip.*;
import java.net.URL;


public class ImageLoader
{
	private final static boolean NOISY = false;
	
	final static ImageObserver IMAGE_OBSERVER = new Panel();
	
	
	
	public static final int getWidth( final Image image ) {
		if( image == null ) {
			return 0;
		}
		return image.getWidth( IMAGE_OBSERVER );
	}
	public static final int getHeight( final Image image ) {
		if( image == null ) {
			return 0;
		}
		return image.getHeight( IMAGE_OBSERVER );
	}

	public static Image loadImage( String fileName ) {
		return loadImage( fileName, IMAGE_OBSERVER );
	}
	private static Image loadImage( String fileName, ImageObserver imageObserver ) {
		
		Image image = _toolkit.getImage( fileName );
		
		_toolkit.prepareImage( image, -1, -1, imageObserver );
		MediaTracker mt = new MediaTracker( (Component)imageObserver );
		mt.addImage( image, 0 );
		try {
			mt.waitForID(0); 
		}
		catch ( InterruptedException ie ) {
			 System.out.println("interrupted trying to wait for the image to load! "+"<:|:>"+"virtuoso.splash.ImageLoader.java.ux:TL-87");
		}
		//System.out.println( "mt.checkID( 0 ); : " + mt.checkID(0) );
		//System.out.println( "mt.isErrorAny(); : " + mt.isErrorAny() );
		return image;
	}
	public static Image loadImage( final URL url ) {
		Image image = _toolkit.getImage( url );
		
		_toolkit.prepareImage( image, -1, -1, IMAGE_OBSERVER );
		MediaTracker mt = new MediaTracker( (Component)IMAGE_OBSERVER );
		mt.addImage( image, 0 );
		try {
			mt.waitForID( 0 ); 
		}
		catch ( InterruptedException ie ) {
			System.out.println("interrupted trying to wait for the image to load! "+"<:|:>"+"virtuoso.splash.ImageLoader.java.ux:TL-103");
		}
		if( NOISY ) {
			//System.out.println( "mt.checkID( 0 ); : " + mt.checkID(0) );
			//System.out.println( "mt.isErrorAny(); : " + mt.isErrorAny() );
			
			if( !mt.isErrorAny() ) {
				System.out.println("Image: " + url + " loaded successfully"+"<:|:>"+"virtuoso.splash.ImageLoader.java.ux:TL-110");
			}
			else {
				System.out.println("Image: " + url + " loaded unsuccesfully"+"<:|:>"+"virtuoso.splash.ImageLoader.java.ux:TL-113");
			}
		}
		return image;
	}
	private static Toolkit _toolkit = Toolkit.getDefaultToolkit();
	
	
	
	public static boolean DEBUG_getPixels;
	public static final Object getPixels( final Image image ) {
		final int width = getWidth( image );
		final int height = getHeight( image );
		
		PixelGrabber pixelGrabber = new PixelGrabber( image, 0, 0, width, height, true );
		
		System.out.println("tp ixels COLOR_MODEL --> pixelGrabber.getColorModel() : " + pixelGrabber.getColorModel() +"<:|:>"+"virtuoso.splash.ImageLoader.java.ux:TL-129");
		
		try {
			pixelGrabber.startGrabbing();
			pixelGrabber.grabPixels();
		}
		catch( Exception exception ) {
			exception.printStackTrace();
		}
		return pixelGrabber.getPixels();
	}
	/**
		Make an offscreen image for drawing purposes...
	*/
	public static Image makeImage( final int width, final int height ) {
		return ((Panel)IMAGE_OBSERVER).createImage( width, height );
	}
	public static Image makeImage( final int width, final int height, final int[] pixels ) {
		MemoryImageSource mis = new MemoryImageSource( width, height, pixels, 0, width );
		Image image = ((Panel)IMAGE_OBSERVER).createImage( mis );
		PixelGrabber pixelGrabber = new PixelGrabber( image, 0, 0, width, height, true );

		return image;
	}


	public final static Image invertImage( final Image image ) {
		if( image == null ) {
			return image;
		}
		final int width = getWidth( image );
		final int height = getHeight( image );
		final int[] imagePixels = (int[])getPixels( image );
		if( imagePixels == null ) {
			return null; // supplied image was not null, but getPixels( image ) returned null!!;
		}
		final int count = imagePixels.length;
		
		
		for( int i = 0; i < count; i++ ) {
			final int argb = imagePixels[ i ];
			
			final int M = 255; // 00000000 00000000 00000000 11111111
			
			final int a = (argb & (M<<24));
			final int r = (255 - ((argb & (M<<16)) >> 16)) << 16;
			final int g = (255 - ((argb & (M<<8)) >> 8))   << 8;
			final int b = (255 - ((argb & (M<<0)) >> 0))   << 0;
			
			final int newRGB = a | r | g | b;
			
			imagePixels[ i ] = newRGB;
		}
		return makeImage( width, height, imagePixels );
	}

}